//---------------------------------------------------------------------------
// Copyright (C) 1997-2009 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMIGCdll
//
// Description:
//
// Implementation
//
// Author: Eliot Muir
// Date:   Sunday, May 11th, 2008 @ 09:29:56 PM
// Revision: $Revision: 1.7 $
//
// Last Edit Date: $Date: 2009-02-18 21:56:47 $
// Source: $Source: /home/cvs/cvsroot/CHM/Attic/CHMIGCdll.cpp,v $
//---------------------------------------------------------------------------
#include "CHMpreCompHead.h"
#pragma hdrstop

#ifdef CHM_NO_PRECOMPILED_HEADERS
#  include <CHM/CHMdll.h>
#  include <CHM/CHMdllClass.h>
#  include <CHM/CHMIGCdll.h>
#  include <CHM/CHMerrorClass.h>
#endif

typedef void* CHMIGCexceptionHandle;

typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCmessageProcessorCreate)(void* StubHandle, void** Handle);
typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCmessageProcessorVoidMethod)(void* Handle);
typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCmessageProcessorConnect)(void* Handle, const char* Host, CHMuint32 Port, const char* pClientID);

typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCmessageProcessorVoidConnectCallback)(void* Handle, void* pCallback);

typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCdllGetString)(void* Handle, const char** ppString);
typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCdllSetString)(void* Handle, const char* ppString);

typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCdllGetBoolean)(void* Handle, CHMboolean* pValue);
typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCdllSetBoolean)(void* Handle, CHMboolean pValue);

typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCdllGetUint32)(void* Handle, CHMuint32* pValue);
typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCdllSetUint32)(void* Handle, CHMuint32 pValue);

typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCdllAddLogMessage)(void* Handle, CHMuint32, const char* );

typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCdllGetStringWithKey)(void* Handle, const char*, const char** ppString);
typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCdllAddStringWithKey)(void* Handle, const char*, const char* pString);
typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCdllRemoveStringWithKey)(void* Handle, const char*);

typedef CHMIGCexceptionHandle (CHM_STD_CALL *CHMIGCexceptionCopy)(void* StubHandle, void* OrigHandle,void** pNewHandle);

class CHMIGCdllPrivate
{
public:
   CHMIGCdllPrivate() : Dll() {}
   ~CHMIGCdllPrivate()
   {
   }

   void loadLibrary(const CHMstring& LibraryName)
   {
      //note that we purposely leak the dll handle by specifying IsOwner is false in the
      //call to loadLibrary below.
      //there is a deadlock problem with IGC.dll that will prevent it from unloading if 
      //FreeLibrary is called on it
      Dll.loadLibrary(LibraryName, false);
   }

   void* getProcAddress(const char* ProcName)
   {   
      // expect it to return NULL if the function does not exist.   
      return Dll.getProcAddress( ProcName );
   }


   CHMIGCmessageProcessorCreate              MessageProcessorCreate;
   CHMIGCmessageProcessorVoidMethod          MessageProcessorDestroy;
   CHMIGCmessageProcessorConnect             MessageProcessorConnect;
   CHMIGCmessageProcessorVoidMethod          MessageProcessorDisconnect;
   CHMIGCdllSetString                        MessageProcessorSetDebugFileName;

   CHMIGCmessageProcessorVoidConnectCallback MessageProcessorConnectMessageReceived;
   CHMIGCmessageProcessorVoidConnectCallback MessageProcessorConnectMessageResubmitPreview;
   CHMIGCmessageProcessorVoidConnectCallback MessageProcessorConnectMessageRequest;
   CHMIGCmessageProcessorVoidConnectCallback MessageProcessorConnectMessageRequestCompleted;
   CHMIGCmessageProcessorVoidConnectCallback MessageProcessorConnectShutdownRequest;
   CHMIGCmessageProcessorVoidConnectCallback MessageProcessorConnectDisconnect;

   //IGCexception
   CHMIGCmessageProcessorVoidMethod ExceptionDestroy;
   CHMIGCmessageProcessorCreate     ExceptionCreate;
   CHMIGCdllGetString               ExceptionGetDescription;
   CHMIGCdllSetString               ExceptionSetDescription;
   CHMIGCexceptionCopy              ExceptionCopy;

   //MessageReceivedResponse
   CHMIGCdllGetBoolean           MessageReceivedResponseGetMessageProcessed;
   CHMIGCdllSetBoolean           MessageReceivedResponseSetMessageProcessed;
   CHMIGCdllGetBoolean           MessageReceivedResponseGetStopChannel;
   CHMIGCdllSetBoolean           MessageReceivedResponseSetStopChannel;
   CHMIGCdllGetBoolean           MessageReceivedResponseGetStopChannelErrorFlag;
   CHMIGCdllSetBoolean           MessageReceivedResponseSetStopChannelErrorFlag;
   CHMIGCdllGetString            MessageReceivedResponseGetStopChannelReason;
   CHMIGCdllSetString            MessageReceivedResponseSetStopChannelReason;
   CHMIGCdllGetBoolean           MessageReceivedResponseGetResendMessage;
   CHMIGCdllSetBoolean           MessageReceivedResponseSetResendMessage;
   CHMIGCdllGetUint32            MessageReceivedResponseGetResendMessageDelay;
   CHMIGCdllSetUint32            MessageReceivedResponseSetResendMessageDelay;
   CHMIGCdllAddLogMessage        MessageReceivedResponseAddLogMessage;
   CHMIGCdllGetStringWithKey     MessageReceivedResponseGetLog;
   CHMIGCdllAddStringWithKey     MessageReceivedResponseAddLog;
   CHMIGCdllRemoveStringWithKey  MessageReceivedResponseRemoveLog;

   //MessageRequestResponse
   CHMIGCdllSetString            MessageRequestResponseSetMessage;
   CHMIGCdllGetString            MessageRequestResponseGetMessage;
   CHMIGCdllSetString            MessageRequestResponseSetMessageId;
   CHMIGCdllGetString            MessageRequestResponseGetMessageId;
   CHMIGCdllSetBoolean           MessageRequestResponseSetIsError;
   CHMIGCdllGetBoolean           MessageRequestResponseGetIsError;
   CHMIGCdllSetBoolean           MessageRequestResponseSetStopChannel;
   CHMIGCdllGetBoolean           MessageRequestResponseGetStopChannel;
   CHMIGCdllSetBoolean           MessageRequestResponseSetStopChannelErrorFlag;
   CHMIGCdllGetBoolean           MessageRequestResponseGetStopChannelErrorFlag;
   CHMIGCdllSetString            MessageRequestResponseSetStopChannelReason;
   CHMIGCdllGetString            MessageRequestResponseGetStopChannelReason;
   CHMIGCdllAddLogMessage        MessageRequestResponseAddLogMessage;
   CHMIGCdllGetStringWithKey     MessageRequestResponseGetLog;
   CHMIGCdllAddStringWithKey     MessageRequestResponseAddLog;
   CHMIGCdllRemoveStringWithKey  MessageRequestResponseRemoveLog;

   CHMdll Dll;
   CHMstring CompleteDllFileName;
};

CHMIGCdll::CHMIGCdll()
{
   pMember = new CHMIGCdllPrivate;
}

CHMIGCdll::~CHMIGCdll()
{
   delete pMember;
}

const char * CHMIGCdll::defaultIgcDllName()
{
#ifdef WIN32
   return "IGC.DLL";
#elif !defined(IGC_SHARED_LIB)
   #error "IGC_SHARED_LIB not defined"
#else
   #define CHMIGC_DLL_STRINGIFY_IMPL(x) #x
   #define CHMIGC_DLL_STRINGIFY(x) CHMIGC_DLL_STRINGIFY_IMPL(x)
   return CHMIGC_DLL_STRINGIFY(IGC_SHARED_LIB);
#endif
}

template <class TLib, class TFunctionType>
static void CHMIGCloadFunction(TLib& Lib,const char* pFunctionDef, TFunctionType& Out, int Line, const char * File )
{
   void* pFunction = Lib.getProcAddress( pFunctionDef );
   if (!pFunction)
   {
      CHMstring Message = CHMstring("Unable to load function '") + pFunctionDef  + "' from library.";
      CHMactivateCondition(Message.c_str(), Line, File);
   }
   Out = (TFunctionType)pFunction;
}

#ifdef WIN32
   #define CHMIGC_LOAD_FUNCTION( Library, FunctionName, FunctionSuffix, FunctionPointerOut ) \
      CHMIGCloadFunction( Library, "_" ## FunctionName ## FunctionSuffix, FunctionPointerOut, __LINE__, __FILE__ )
#else
   #define CHMIGC_LOAD_FUNCTION( Library, FunctionName, FunctionSuffix, FunctionPointerOut ) \
      CHMIGCloadFunction( Library, FunctionName, FunctionPointerOut, __LINE__, __FILE__ )
#endif

void CHMIGCloadDll(CHMIGCdllPrivate& Library, const char* pPath)
{
   Library.loadLibrary(pPath);

   //IGCmessageProcessor
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageProcessorCreate", "@8",Library.MessageProcessorCreate);   
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageProcessorDestroy", "@4",Library.MessageProcessorDestroy);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageProcessorconnect", "@16",Library.MessageProcessorConnect);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageProcessordisconnect", "@4",Library.MessageProcessorDisconnect);

   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageProcessorSetOnMessageReceived", "@8",Library.MessageProcessorConnectMessageReceived);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageProcessorSetOnMessageResubmitPreview", "@8",Library.MessageProcessorConnectMessageResubmitPreview);

   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageProcessorSetOnMessageRequest", "@8",Library.MessageProcessorConnectMessageRequest);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageProcessorSetOnMessageRequestCompleted", "@8",Library.MessageProcessorConnectMessageRequestCompleted);
   
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageProcessorSetOnShutdownRequest", "@8",Library.MessageProcessorConnectShutdownRequest);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageProcessorSetOnDisconnect", "@8",Library.MessageProcessorConnectDisconnect);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageProcessorsetDebugFileName", "@8",Library.MessageProcessorSetDebugFileName);

   //IGCexception
   CHMIGC_LOAD_FUNCTION(Library,"IGCexceptionCreate", "@8",Library.ExceptionCreate);
   CHMIGC_LOAD_FUNCTION(Library,"IGCexceptionDestroy", "@4",Library.ExceptionDestroy);
   CHMIGC_LOAD_FUNCTION(Library,"IGCexceptionGetDescription", "@8",Library.ExceptionGetDescription);
   CHMIGC_LOAD_FUNCTION(Library,"IGCexceptionSetDescription", "@8",Library.ExceptionSetDescription);
   CHMIGC_LOAD_FUNCTION(Library,"IGCexceptionCopy", "@12",Library.ExceptionCopy);

   //IGCmessageReceivedResponse
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseGetMessageProcessed", "@8",Library.MessageReceivedResponseGetMessageProcessed);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseSetMessageProcessed", "@8",Library.MessageReceivedResponseSetMessageProcessed );
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseGetStopChannel", "@8",Library.MessageReceivedResponseGetStopChannel);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseSetStopChannel", "@8",Library.MessageReceivedResponseSetStopChannel);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseGetStopChannelErrorFlag", "@8",Library.MessageReceivedResponseGetStopChannelErrorFlag);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseSetStopChannelErrorFlag", "@8",Library.MessageReceivedResponseSetStopChannelErrorFlag);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseGetStopChannelReason", "@8",Library.MessageReceivedResponseGetStopChannelReason);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseSetStopChannelReason", "@8",Library.MessageReceivedResponseSetStopChannelReason);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseGetResendMessage", "@8",Library.MessageReceivedResponseGetResendMessage);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseSetResendMessage", "@8",Library.MessageReceivedResponseSetResendMessage);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseGetResendMessageDelay", "@8",Library.MessageReceivedResponseGetResendMessageDelay);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseSetResendMessageDelay", "@8",Library.MessageReceivedResponseSetResendMessageDelay);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseAddLogMessage", "@12",Library.MessageReceivedResponseAddLogMessage);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponselogValue", "@12",Library.MessageReceivedResponseGetLog);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseAddLogValue", "@12",Library.MessageReceivedResponseAddLog);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageReceivedResponseRemoveLogValue", "@8",Library.MessageReceivedResponseRemoveLog);

   //IGCmessageRequestResponse
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseSetOutboundMessage", "@8",Library.MessageRequestResponseSetMessage);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseGetOutboundMessage", "@8",Library.MessageRequestResponseGetMessage);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseSetMessageId", "@8",Library.MessageRequestResponseSetMessageId);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseGetMessageId", "@8",Library.MessageRequestResponseGetMessageId);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseSetMessageIsError", "@8",Library.MessageRequestResponseSetIsError);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseGetMessageIsError", "@8",Library.MessageRequestResponseGetIsError);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseSetStopChannel", "@8",Library.MessageRequestResponseSetStopChannel);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseGetStopChannel", "@8",Library.MessageRequestResponseGetStopChannel);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseSetStopChannelErrorFlag", "@8",Library.MessageRequestResponseSetStopChannelErrorFlag);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseGetStopChannelErrorFlag", "@8",Library.MessageRequestResponseGetStopChannelErrorFlag);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseSetStopChannelReason", "@8",Library.MessageRequestResponseSetStopChannelReason);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseGetStopChannelReason", "@8",Library.MessageRequestResponseGetStopChannelReason);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseAddLogMessage", "@12",Library.MessageRequestResponseAddLogMessage);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponselogValue", "@12",Library.MessageRequestResponseGetLog);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseAddLogValue", "@12",Library.MessageRequestResponseAddLog);
   CHMIGC_LOAD_FUNCTION(Library,"IGCmessageRequestResponseRemoveLogValue", "@8",Library.MessageRequestResponseRemoveLog);

}

void CHMIGCdll::loadIgcDll(const char* pCompleteDllPath)
{
   CHMIGCloadDll(*pMember, pCompleteDllPath);
}

void CHMIGCcheckCall(CHMIGCexceptionHandle ExceptionHandle, CHMIGCmessageProcessorVoidMethod pDestroy, CHMIGCdllGetString pGetDescription)
{
   if (ExceptionHandle)
   {
      const char* pDescription = 0;
      (*pGetDescription)(ExceptionHandle, &pDescription );
      CHMstring Description = pDescription;
      (*pDestroy)(ExceptionHandle);
      CHMactivateCondition(Description.c_str(), __LINE__, __FILE__ ); // TODO - file and line are useless here
   }
}

void* CHMIGCdll::messageProcessorCreate(void* StubHandle)
{
   void* ProcessorHandle = 0;
   CHMIGCcheckCall((*pMember->MessageProcessorCreate)(StubHandle, &ProcessorHandle), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);
   return ProcessorHandle;
}

void CHMIGCdll::messageProcessorDestroy(void* ProcessorHandle)
{
   // We need to clean up exceptions that are produced though to avoid a memory leak
   // but we don't throw here because this function is used in destructors
   CHMIGCexceptionHandle Result = (*pMember->MessageProcessorDestroy)(ProcessorHandle);
   if (Result)
   {
      //do not check result...this must always succeed
      (*pMember->ExceptionDestroy)(Result);
   }
}

void CHMIGCdll::messageProcessorConnect(void* ProcessorHandle, const char* Host, CHMuint32 Port, const char* pClientId)
{
   CHMIGCcheckCall((*pMember->MessageProcessorConnect)(ProcessorHandle, Host, Port, pClientId), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);
}

void* CHMIGCdll::messageProcessorConnectNoThrow(void* ProcessorHandle, const char* Host, CHMuint32 Port, const char* pClientId)
{
   return (*pMember->MessageProcessorConnect)(ProcessorHandle, Host, Port, pClientId);
}

void CHMIGCdll::messageProcessorSetDebugFileName(void* ProcessorHandle, const char* FileName)
{
   CHMIGCcheckCall((*pMember->MessageProcessorSetDebugFileName)(ProcessorHandle,FileName), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

void CHMIGCdll::messageProcessorDisconnect(void* ProcessorHandle)
{
   CHMIGCcheckCall((*pMember->MessageProcessorDisconnect)(ProcessorHandle), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);   
}

void CHMIGCdll::connectForMessageReceived(void* pProcessorHandle, TCHMIGCmessageReceivedCallback pCallback)
{
   CHMIGCcheckCall((*pMember->MessageProcessorConnectMessageReceived)(pProcessorHandle,(void*)pCallback), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);   
}

void CHMIGCdll::connectForMessageResubmitPreview(void* pProcessorHandle, TCHMIGCmessageResubmitPreviewCallback pCallback)
{
   CHMIGCcheckCall((*pMember->MessageProcessorConnectMessageResubmitPreview)(pProcessorHandle,(void*)pCallback), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

void CHMIGCdll::connectForMessageRequest(void* pProcessorHandle, TCHMIGCmessageRequestCallback pCallback)
{
   CHMIGCcheckCall((*pMember->MessageProcessorConnectMessageRequest)(pProcessorHandle,(void*)pCallback), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

void CHMIGCdll::connectForMessageRequestCompleted(void* pProcessorHandle, TCHMIGCmessageRequestCompletedCallback pCallback)
{
   CHMIGCcheckCall((*pMember->MessageProcessorConnectMessageRequestCompleted)(pProcessorHandle,(void*)pCallback), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

void CHMIGCdll::connectForShutdownRequest(void* pProcessorHandle, TCHMIGCshutdownRequestCallback pCallback)
{
   CHMIGCcheckCall((*pMember->MessageProcessorConnectShutdownRequest)(pProcessorHandle,(void*)pCallback), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

void CHMIGCdll::connectForDisconnect(void* pProcessorHandle, TCHMIGCdisconnectCallback pCallback)
{
   CHMIGCcheckCall((*pMember->MessageProcessorConnectDisconnect)(pProcessorHandle,(void*)pCallback), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

void CHMIGCdll::messageReceivedResponseSetProcessed( void* pResponseHandle, CHMboolean NewValue )
{
   CHMIGCcheckCall((*pMember->MessageReceivedResponseSetMessageProcessed)(pResponseHandle,NewValue), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

CHMboolean CHMIGCdll::messageReceivedGetResponseProcessed( void* pResponseHandle )
{
   CHMboolean Result = false;
   CHMIGCcheckCall((*pMember->MessageReceivedResponseGetMessageProcessed)(pResponseHandle,&Result), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
   return Result;
}

void CHMIGCdll::messageReceivedResponseSetStopChannel( void* pResponseHandle, CHMboolean NewValue)
{
   CHMIGCcheckCall((*pMember->MessageReceivedResponseSetStopChannel)(pResponseHandle,NewValue), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

CHMboolean CHMIGCdll::messageReceivedResponseGetStopChannel( void* pResponseHandle )
{
   CHMboolean Result = 0;
   CHMIGCcheckCall((*pMember->MessageReceivedResponseGetStopChannel)(pResponseHandle,&Result), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
   return Result;
}

void CHMIGCdll::messageReceivedResponseSetStopChannelErrorFlag( void* pResponseHandle, CHMboolean NewValue)
{
   CHMIGCcheckCall((*pMember->MessageReceivedResponseSetStopChannelErrorFlag)(pResponseHandle,NewValue), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

CHMboolean CHMIGCdll::messageReceivedResponseGetStopChannelErrorFlag( void* pResponseHandle )
{
   CHMboolean Result = 0;
   CHMIGCcheckCall((*pMember->MessageReceivedResponseGetStopChannelErrorFlag)(pResponseHandle,&Result), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
   return Result;
}

void CHMIGCdll::messageReceivedResponseSetStopChannelReason( void* pResponseHandle, const char* NewValue)
{
   CHMIGCcheckCall((*pMember->MessageReceivedResponseSetStopChannelReason)(pResponseHandle,NewValue), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

const char* CHMIGCdll::messageReceivedResponseGetStopChannelReason( void* pResponseHandle )
{
   const char* Result = 0;
   CHMIGCcheckCall((*pMember->MessageReceivedResponseGetStopChannelReason)(pResponseHandle,&Result), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
   return Result;
}

void CHMIGCdll::messageReceivedResponseSetResendMessage( void* pResponseHandle, CHMboolean NewValue)
{
   CHMIGCcheckCall((*pMember->MessageReceivedResponseSetResendMessage)(pResponseHandle,NewValue), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

CHMboolean CHMIGCdll::messageReceivedResponseGetResendMessage( void* pResponseHandle )
{
   CHMboolean Result = 0;
   CHMIGCcheckCall((*pMember->MessageReceivedResponseGetResendMessage)(pResponseHandle,&Result), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
   return Result;
}

void CHMIGCdll::messageReceivedResponseSetResendMessageDelay( void* pResponseHandle, CHMuint32 NewValue)
{
   CHMIGCcheckCall((*pMember->MessageReceivedResponseSetResendMessageDelay)(pResponseHandle,NewValue), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

CHMuint32 CHMIGCdll::messageReceivedResponseGetResendMessageDelay( void* pResponseHandle )
{
   CHMuint32 Result = 0;
   CHMIGCcheckCall((*pMember->MessageReceivedResponseGetResendMessageDelay)(pResponseHandle,&Result), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
   return Result;
}

void CHMIGCdll::messageReceivedResponseAddLogMessage( void* pResponseHandle, CHMuint32 LogMessageType, const char* LogMessage )
{
   CHMIGCcheckCall((*pMember->MessageReceivedResponseAddLogMessage)(pResponseHandle,LogMessageType, LogMessage), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
}

const char* CHMIGCdll::messageReceivedResponseGetLog
(
   void* pResponseHandle, 
   const char* pKey
)
{
   const char* pResult = 0;
   CHMIGCcheckCall((*pMember->MessageReceivedResponseGetLog)(pResponseHandle,pKey,&pResult), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
   return pResult;
}

void CHMIGCdll::messageReceivedResponseAddLog
(
   void* pResponseHandle, 
   const char* pKey, 
   const char* pValue
)
{
   CHMIGCcheckCall((*pMember->MessageReceivedResponseAddLog)(pResponseHandle,pKey,pValue), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

void CHMIGCdll::messageReceivedResponseRemoveLog
(
   void* pResponseHandle, 
   const char* pKey
)
{
   CHMIGCcheckCall((*pMember->MessageReceivedResponseRemoveLog)(pResponseHandle,pKey), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

void CHMIGCdll::messageRequestResponseAddLogMessage
(
   void* pResponseHandle, 
   CHMuint32 LogMessageType,
   const char* LogMessage
)
{
   CHMIGCcheckCall((*pMember->MessageRequestResponseAddLogMessage)(pResponseHandle,LogMessageType,LogMessage), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

const char* CHMIGCdll::messageRequestResponseGetLog
(
   void* pResponseHandle, 
   const char* pKey
)
{
   const char* pResult = 0;
   CHMIGCcheckCall((*pMember->MessageRequestResponseGetLog)(pResponseHandle,pKey,&pResult), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
   return pResult;
}

void CHMIGCdll::messageRequestResponseAddLog
(
   void* pResponseHandle, 
   const char* pKey, 
   const char* pValue
)
{
   CHMIGCcheckCall((*pMember->MessageRequestResponseAddLog)(pResponseHandle,pKey,pValue), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

void CHMIGCdll::messageRequestResponseRemoveLog
(
   void* pResponseHandle, 
   const char* pKey
)
{
   CHMIGCcheckCall((*pMember->MessageRequestResponseRemoveLog)(pResponseHandle,pKey), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);      
}

void CHMIGCdll::messageRequestResponseSetMessageId( void* pResponseHandle, const char* pMessageId )
{
   CHMIGCcheckCall((*pMember->MessageRequestResponseSetMessageId)(pResponseHandle,pMessageId), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
}

const char* CHMIGCdll::messageRequestResponseGetMessageId( void* pResponseHandle )
{
   const char* pValue = 0;
   CHMIGCcheckCall((*pMember->MessageRequestResponseGetMessageId)(pResponseHandle,&pValue), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
   return pValue;
}

void CHMIGCdll::messageRequestResponseSetMessage( void* pResponseHandle, const char* pMessage)
{
   CHMIGCcheckCall((*pMember->MessageRequestResponseSetMessage)(pResponseHandle,pMessage), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
}

const char* CHMIGCdll::messageRequestResponseGetMessage( void* pResponseHandle )
{
   const char* pValue = 0;
   CHMIGCcheckCall((*pMember->MessageRequestResponseGetMessage)(pResponseHandle,&pValue), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
   return pValue;
}

CHMboolean CHMIGCdll::messageRequestResponseGetIsError( void* pResponseHandle )
{
   CHMboolean Value = false;
   CHMIGCcheckCall((*pMember->MessageRequestResponseGetIsError)(pResponseHandle,&Value), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
   return Value;
}

void CHMIGCdll::messageRequestResponseSetIsError( void* pResponseHandle, CHMboolean Value)
{
   CHMIGCcheckCall((*pMember->MessageRequestResponseSetIsError)(pResponseHandle,Value), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
}

CHMboolean CHMIGCdll::messageRequestResponseGetStopChannel( void* pResponseHandle )
{
   CHMboolean Value = false;
   CHMIGCcheckCall((*pMember->MessageRequestResponseGetStopChannel)(pResponseHandle,&Value), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
   return Value;
}

void CHMIGCdll::messageRequestResponseSetStopChannel( void* pResponseHandle, CHMboolean Value )
{
   CHMIGCcheckCall((*pMember->MessageRequestResponseSetStopChannel)(pResponseHandle,Value), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
}

CHMboolean CHMIGCdll::messageRequestResponseGetStopChannelErrorFlag( void* pResponseHandle )
{
   CHMboolean Value = false;
   CHMIGCcheckCall((*pMember->MessageRequestResponseGetStopChannelErrorFlag)(pResponseHandle,&Value), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
   return Value;
}

void CHMIGCdll::messageRequestResponseSetStopChannelErrorFlag( void* pResponseHandle, CHMboolean Value )
{
   CHMIGCcheckCall((*pMember->MessageRequestResponseSetStopChannelErrorFlag)(pResponseHandle,Value), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
}

const char* CHMIGCdll::messageRequestResponseGetStopChannelReason( void* pResponseHandle )
{
   const char* Value = 0;
   CHMIGCcheckCall((*pMember->MessageRequestResponseGetStopChannelReason)(pResponseHandle,&Value), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
   return Value;
}

void CHMIGCdll::messageRequestResponseSetStopChannelReason( void* pResponseHandle, const char* Value )
{
   CHMIGCcheckCall((*pMember->MessageRequestResponseSetStopChannelReason)(pResponseHandle,Value), pMember->ExceptionDestroy, pMember->ExceptionGetDescription);         
}

void* CHMIGCdll::exceptionCreate(void* pStubHandle)
{
   void* pNewException = 0;
   void* pResult = (*pMember->ExceptionCreate)(pStubHandle, &pNewException );
   CHMPRECONDITION( pResult == 0 )
   return pNewException;
}

void CHMIGCdll::exceptionDestroy(void* pExceptionHandle)
{
   (*pMember->ExceptionDestroy)(pExceptionHandle);
}

const char* CHMIGCdll::exceptionGetDescription(void* pExceptionHandle)
{
   const char* pDescription = 0;
   void* pResult = (*pMember->ExceptionGetDescription)(pExceptionHandle, &pDescription );
   CHMPRECONDITION( pResult == 0 )
   return pDescription;
}

void CHMIGCdll::exceptionSetDescription(void* pExceptionHandle, const char* pDesc)
{
   void* pResult = (*pMember->ExceptionSetDescription)(pExceptionHandle, pDesc );
   CHMPRECONDITION( pResult == 0 )
}

void* CHMIGCdll::exceptionCopy(void* pStubHandle, void* pOrigException)
{
   void* pNewException = 0;
   void* pResult = (*pMember->ExceptionCopy)(pStubHandle, pOrigException, &pNewException);
   CHMPRECONDITION( pResult == 0 )
   return pNewException;
}

CHMstring CHMIGCdll::completeDllPath() const
{
   return pMember->Dll.libraryName();
}
