#include <CHM/CHMpreCompHead.h>
#pragma hdrstop
//------------------------------------------------------------------------
// Copyright (C) 1997-2005 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMengineClass
//
// Description:
//
// Implementation
//
// Author: Eliot Muir
// Date:   Wed 09/09/1998 
// Revision: $Revision: 1.66 $
//
// Last Edit Date: $Date: 2007-03-26 17:11:03 $
// Source: $Source: /home/cvs/cvsroot/CHM/CHMengineClass.cpp,v $
//---------------------------------------------------------------------------

#ifdef CHM_NO_PRECOMPILED_HEADERS
#   include <CHM/CHMengineClass.h>
#   include <CHM/CHMengine.h>
#   include <CHM/CHMerrorClass.h>
#   include <CHM/CHMoutputLog.h>
#   include <CHM/CHMwrapper.h>
#   include <CHM/CHMtransport.h>
#   include <CHM/CHMstring.h>
#   include <CHM/CHMtableClass.h>
#endif

class CHMenginePrivate
{
public:
   CHMenginePrivate()
   : Handle(NULL), 
     pLog(NULL), 
     pTransport(NULL),
     pWrapper(NULL),
     pNewLine(CHM_TEXT("\r\n"))
   {
      {
         CHM_CHECK_CALL(CHMengineCreate(&Handle));
      }
      {
         CHM_CHECK_CALL(CHMengineAddRef(Handle));
      }
   }
   
   CHMenginePrivate(CHMengineHandle handle)
   : Handle(handle), 
     pLog(NULL), 
     pTransport(NULL),
     pWrapper(NULL),
     pNewLine(CHM_TEXT("\r\n"))
   {
      {
         CHM_CHECK_CALL(CHMengineAddRef(Handle));
      }
   }
   
   ~CHMenginePrivate() 
   { 
      delete pLog;
      delete pWrapper;
      delete pTransport;
      CHM_CHECK_CALL(CHMengineRelease(Handle));
   }

   static void CHM_STD_CALL OutputLog
   (
      void* CallbackObjectPtr, 
      const char* pLine
   );

   CHMengineHandle Handle;
   CHMoutputLog* pLog;
   CHMwrapper*     pWrapper;
   CHMtransport*   pTransport;
   const CHMchar*     pNewLine;
};

void CHM_STD_CALL CHMenginePrivate::OutputLog
(
   void* CallbackObjectPtr, 
   const char* pLine
)
{
   CHMoutputLog* pThisLog = ((CHMenginePrivate*)CallbackObjectPtr)->pLog;
   if (pThisLog)
   {
      pThisLog->OutputLine(pLine);
   }
}

CHMengine::CHMengine() : pMember(NULL)
{
   pMember = new CHMenginePrivate;
}

CHMengine::CHMengine(CHMengineHandle handle) : pMember(NULL)
{
   pMember = new CHMenginePrivate(handle);
}

CHMengine::~CHMengine()
{
   delete pMember;
}

void CHMengine::SetLog(CHMoutputLog* pLog)
{
   delete pMember->pLog;
   pMember->pLog = pLog;
   CHM_CHECK_CALL(CHMengineSetLogCallBackPerEngine(pMember->Handle, pMember, CHMenginePrivate::OutputLog));
   pMember->pNewLine = pLog->NewLine();
}

CHMoutputLog* CHMengine::Log()
{
   return pMember->pLog;
}

void CHMengine::Load(const CHMchar* pFileName)
{
   CHM_CHECK_CALL(CHMengineLoadT(pMember->Handle, pFileName));
}

const CHMengineHandle CHMengine::EngineHandle() const
{
   return pMember->Handle;
}

void CHMengine::SetRejectBadSegmentGrammarFlag
(
   CHMboolean Flag
)
{
   CHM_CHECK_CALL(CHMengineSetRejectBadSegmentGrammarFlag(pMember->Handle, Flag));
}

void CHMengine::OutputLine(const CHMchar* pLine) const
{
   if (NULL != pMember->pLog)
   {
      pMember->pLog->OutputLine(pLine);
   }
}

//void CHMengine::OutputStream(IOstrstream& Stream) const
//{
//   if (NULL != pMember->pLog)
//   {
//      pMember->pLog->OutputStream(Stream);
//   }
//}

const CHMchar* CHMengine::NewLine() const
{
   return pMember->pNewLine;
}

void CHMengine::SetTransport(CHMtransport* pTransport)
{
   delete pMember->pTransport;
   pMember->pTransport = pTransport;
   pMember->pTransport->SetEngine(this);
}

CHMtransport* CHMengine::Transport()
{
   return pMember->pTransport;
}

const CHMtransport* CHMengine::Transport() const
{
   return pMember->pTransport;
}

void CHMengine::SetWrapper(CHMwrapper* pWrapper)
{
   delete pMember->pWrapper;
   pMember->pWrapper = pWrapper;
   pMember->pWrapper->SetEngine(this);
}

CHMwrapper* CHMengine::Wrapper()
{
   return pMember->pWrapper;
}

const CHMwrapper* CHMengine::Wrapper() const
{
   return pMember->pWrapper;
}

void CHMengine::InitiateMessage(const CHMchar* FlatWire)
{
   CHM_ASSERT(Wrapper() != NULL);

   Wrapper()->SendResponseMessage(FlatWire, 0);    
}

void CHMengine::ReplyToMessage
(
   const CHMchar* FlatWire, 
   size_t TransportIndex
)
{
   CHM_ASSERT(Wrapper() != NULL);

   Wrapper()->SendResponseMessage(FlatWire, TransportIndex); 
}

void CHMengine::ParseMessageDynamically
(
   const CHMchar* pFlatWire,
   CHMtable&      Table,
   size_t&        MessageIndex
) const
{
   CHMresult Ok = CHM_DLL_CALL(CHMengineParseMessage(EngineHandle(),
                              pFlatWire, &MessageIndex, Table.TableHandle()));

   if (Ok != CHM_OK)
   {
      throw CHMerror(Ok, __LINE__, __FILE__);
   }
}

size_t CHMengine::CountOfMessage() const
{
   size_t CountOfMessage;
   CHM_CHECK_CALL(CHMengineCountOfMessage(EngineHandle(), &CountOfMessage)); 
   return CountOfMessage;
}

const CHMchar* CHMengine::MessageName(size_t MessageIndex) const
{
   const CHMchar* pMessageName;
   CHM_CHECK_CALL(CHMengineMessageNameT(EngineHandle(), MessageIndex, &pMessageName));
   return pMessageName;
}
 
size_t CHMengine::CountOfTable() const
{
   size_t CountOfTable;
   CHM_CHECK_CALL(CHMengineGetCountOfTable(EngineHandle(), &CountOfTable)); 
   return CountOfTable;
}

CHMtableHandle CHMengine::TableHandle(size_t TableIndex) const
{
   CHMtableHandle handle;
   CHM_CHECK_CALL(CHMengineGetTable(EngineHandle(), TableIndex, &handle)); 
   return handle;
}

CHMtableHandle CHMengine::MessageHandle(size_t MessageIndex) const
{
   CHMmessageHandle handle;
   CHM_CHECK_CALL(CHMengineGetMessage(EngineHandle(), MessageIndex, &handle)); 
   return handle;
}

/*
const CHMchar* CHMengine::TableName(size_t TableIndex) const
{
   const CHMchar* pTableName;
   CHM_CHECK_CALL(CHMengineTableName(EngineHandle(), TableIndex, &pTableName));
   return pTableName;
}

size_t CHMengine::CountOfColumn(size_t TableIndex) const
{
   size_t ColumnCount;
   CHM_CHECK_CALL(CHMengineCountOfColumn(EngineHandle(), TableIndex, &ColumnCount)); 
   return ColumnCount;
}

size_t CHMengine::TypeOfColumn(size_t TableIndex, size_t ColumnIndex) const
{
   size_t ColumnType;
   CHM_CHECK_CALL(CHMengineTypeOfColumn(EngineHandle(), TableIndex, ColumnIndex, &ColumnType)); 
   return ColumnType;
}
*/
  
void CHMengine::ParseMessage
(
   const CHMchar* pFlatWire, 
   size_t TransportIndex
)
{
   throw CHMerror(CHM_TEXT("The ParseMessage method should be overridden."), __LINE__, __FILE__, 0);
}

size_t CHMengine::AckMessageIndex() const
{
   size_t MessageIndex;
   CHM_CHECK_CALL(CHMengineGetAckMessageIndex(EngineHandle(), &MessageIndex));
   return MessageIndex;
}

const CHMchar* CHMengine::transformMessage
(
   const CHMchar* pMessage,
   size_t InConfigIndex,
   size_t OutConfigIndex,
   size_t& MessageIndex
)
{
   const CHMchar* pOutMessage;
   CHM_CHECK_CALL(CHMengineTransformMessageT(EngineHandle(),
                                            pMessage,
                                            InConfigIndex,
                                            OutConfigIndex,
                                            &pOutMessage,
                                            &MessageIndex,
                                            0));
   return pOutMessage;                                         
}

size_t CHMengine::CurrentConfigurationIndex() const 
{
   size_t Index;
   CHM_CHECK_CALL(CHMengineGetCurrentConfig(EngineHandle(), &Index));
   return Index;
}

void CHMengine::SetCurrentConfigurationIndex(size_t Index)
{
   CHM_CHECK_CALL(CHMengineSetCurrentConfig(EngineHandle(), Index));  
}

size_t CHMengine::CountOfConfiguration() const
{
   size_t ConfigurationCount;
   CHM_CHECK_CALL(CHMengineCountOfConfig(EngineHandle(), &ConfigurationCount));
   return ConfigurationCount;   
}

const CHMchar* CHMengine::ConfigurationName(size_t ConfigurationIndex) const
{
   const CHMchar* pConfigurationName;
   CHM_CHECK_CALL(CHMengineConfigName(EngineHandle(), ConfigurationIndex, &pConfigurationName));
   return pConfigurationName;
}

size_t CHMengine::InConfigurationIndex() const
{
   size_t Index;
   CHM_CHECK_CALL(CHMengineGetIncomingConfigIndex(EngineHandle(), &Index));   
   return Index;
}

size_t CHMengine::OutConfigurationIndex() const
{
   size_t Index;
   CHM_CHECK_CALL(CHMengineGetOutgoingConfigIndex(EngineHandle(), &Index));
   return Index;
}


size_t CHMengine::IgnoreMessageIndex() const
{
   size_t Index;
   CHM_CHECK_CALL(CHMengineGetIgnoreMessageIndex(EngineHandle(), &Index));
   return Index;
}

const char* CHMengine::translateMessageToXml
(
   const char* pFlatWire
)
{
   const char* pResult;
   CHM_CHECK_CALL(CHMengineTranslateMessageToHl7StandardXml(EngineHandle(),pFlatWire, &pResult));
   return pResult;
}

const char* CHMengine::translateXmlToMessage
(
   const char* pXml
)
{
  const char* pResult;
   CHM_CHECK_CALL(CHMengineTranslateHl7StandardXmlToMessage(EngineHandle(),pXml, &pResult));
   return pResult;
}

void CHMengine::generateXsd( const char* pFilename )
{
   CHM_CHECK_CALL(CHMengineGetTibcoSchema(EngineHandle(), pFilename));
}

const CHMchar* CHMengine::generateAck(const CHMchar* pMessage)
{
   const CHMchar* pOutAck;
   CHM_CHECK_CALL(CHMengineGenerateAckFromMessage(EngineHandle(), pMessage, &pOutAck));
   return pOutAck;
}

void CHMengine::setPythonKeyValuePair
(
   const char* pKey, 
   const char* pValue
)
{
   CHM_CHECK_CALL(CHMengineSetPythonKeyValuePair(EngineHandle(),pKey,pValue));
}

void CHMengine::extendPythonLibraryPath
(
   const char* pPath 
)
{
   CHM_CHECK_CALL(CHMengineExtendPythonLibraryPath(EngineHandle(),pPath));
}

size_t CHMengine::majorVersion() const
{
  size_t MajorVersion;
  CHM_CHECK_CALL(CHMengineGetMajorVersion(EngineHandle(),&MajorVersion));
  return MajorVersion;
}

size_t CHMengine::minorVersion() const
{
  size_t MinorVersion;
  CHM_CHECK_CALL(CHMengineGetMinorVersion(EngineHandle(),&MinorVersion));
  return MinorVersion;
}

size_t CHMengine::buildNumber() const
{
  size_t BuildNumber;
  CHM_CHECK_CALL(CHMengineGetBuildNumber(EngineHandle(),&BuildNumber));
  return BuildNumber;
}

const CHMchar* CHMengine::versionString() const
{
  const CHMchar* pVersionString;
  CHM_CHECK_CALL(CHMengineGetVersionString(EngineHandle(), &pVersionString));
  return pVersionString;
}


void CHMengine::setDefaultDatabase
(
   const char* pAPI, 
   const char* pName,
   const char* pUserName,
   const char* pPassword
)
{
   CHM_CHECK_CALL
      (CHMengineSetDefaultDatabase
         (
          EngineHandle(), 
          pAPI, 
          pName, 
          pUserName, 
          pPassword
         )
      );
}

CHMboolean CHMengine::IsUsingPassthruMapping() const
{
   CHMboolean ReturnValue = 0;
   CHM_CHECK_CALL(CHMengineIsUsingPassthruMapping(EngineHandle(), &ReturnValue));
   return ReturnValue;
}
