#include <CHM/CHMpreCompHead.h>
#pragma hdrstop

#ifdef CHM_NO_PRECOMPILED_HEADERS
#   include <CHM/CHMerrorClass.h>
#   include <CHM/CHMerror.h>
#   include <string.h>
#   include <stdlib.h>
#endif

#if defined(_USE_MFC_EXCEPTIONS)

IMPLEMENT_DYNAMIC(CHMerror, CException)

CHMerror::CHMerror
(
   const CHMchar* Message, 
   long Line, 
   const char* File, 
   long Code
) : CException(TRUE)
{
   m_Handle = NULL;
   m_pDescription = CHMstrdup(Message);
   m_Code = Code;
   m_File = CHMstrdup(File);
   m_Line = Line;
}

CHMerror::CHMerror(const CHMerror& Error)
 : m_File(NULL), m_pDescription(NULL)
{
   m_Handle = Error.m_Handle;
   CHM_DLL_CALL(CHMreferenceAddRef(m_Handle));
   m_pDescription = CHMstrdup(Error.m_pDescription);
   m_Code = Error.m_Code;
   m_File = CHMstrdup(Error.m_File);
   m_Line = Error.m_Line;
}

CHMerror::CHMerror
(
   const CHMresult Handle, 
   long Line, 
   const char* File
) : CException(TRUE)
{
   m_File = CHMstrdup(File);
   m_Line = Line;
   if (Handle == (void*)CHM_UNKNOWN_EXCEPTION)
   {
      m_Handle = NULL;
      m_Code = -1;
      m_pDescription = CHMstrdup("Call failed.");
   }
   else
   {
      m_Handle = Handle;
      CHM_DLL_CALL(CHMreferenceAddRef(m_Handle));
      CHM_DLL_CALL(CHMerrorGetCode(m_Handle, &m_Code));
      CHMchar* Description;
      CHM_DLL_CALL(CHMerrorGetDescriptionT(m_Handle, &Description));
      m_pDescription = CHMstrdup(Description);
   }   
}
CHMerror::~CHMerror()
{
   CHM_DLL_CALL(CHMreferenceRelease(m_Handle));
   m_Handle = NULL;
   free(m_pDescription);
   free(m_File);
}

BOOL CHMerror::GetErrorMessage
(
   LPTSTR lpszError, 
   UINT nMaxError,
    PUINT pnHelpContext
)
{
   if (NULL != pnHelpContext)
   {
      *pnHelpContext = UINT(m_Code);
   }
   CHMstrncpy(lpszError, Dump.c_str(), nMaxError -1);
   if (strlen(Dump.c_str()) >= nMaxError)
   {
      // terminate string
      lpszError[nMaxError] = NULL;
   }
   return TRUE;
}

//void CHMerror::printOn(ostream& Stream) const
//{
//   Stream << "Error Code : " << Code() << endl
//          << "Description: " << Description() << endl
//          << "Line       : " << m_Line << endl
//          << "File       : " << m_File << endl;
//}

const long CHMerror::Code() const
{
   return m_Code;
}

CHMchar* CHMerror::Description() const
{
   return m_pDescription;
}

void CHMerror::Parameter(const CHMchar* pKey, const CHMchar** ppValue) const
{
   CHM_DLL_CALL(CHMerrorGetParameter(m_Handle, pKey, ppValue));
}

#else // if ! defined( _USE_MFC_EXCEPTIONS )

CHMerror::CHMerror(const CHMchar* Message, long Line, const char* File, long Code)
 : m_File(NULL), m_pDescription(NULL)
{
   m_Handle = NULL;
   m_pDescription = CHMstrdup(Message);
   m_Code = Code;
   m_File = _strdup(File);
   m_Line = Line;
}

CHMerror::CHMerror(const CHMerror& Error)
 : m_File(NULL), m_pDescription(NULL)
{
   m_Handle = Error.m_Handle;
   CHM_DLL_CALL(CHMreferenceAddRef(m_Handle));
   m_pDescription = CHMstrdup(Error.m_pDescription);
   m_Code = Error.m_Code;
   m_File = _strdup(Error.m_File);
   m_Line = Error.m_Line;
}

CHMerror::CHMerror(const CHMresult Handle, long Line, const char* File)
 : m_File(NULL), m_pDescription(NULL)
{
   m_File = _strdup(File);
   m_Line = Line;
   if (Handle == (void*)CHM_UNKNOWN_EXCEPTION)
   {
      m_Handle = NULL;
      m_Code = -1;
      m_pDescription = CHMstrdup(CHM_TEXT("Call failed."));
   }
   else
   {
      m_Handle = Handle;
      CHM_DLL_CALL(CHMreferenceAddRef(m_Handle));
      unsigned long UCode;
      CHM_DLL_CALL(CHMerrorGetCode(m_Handle, &UCode));
      m_Code = (long)UCode;
      const CHMchar* Description;
      CHM_DLL_CALL(CHMerrorGetDescriptionT(m_Handle, &Description));
      m_pDescription = CHMstrdup(Description);
   }
}

CHMerror::~CHMerror()
{
   CHM_DLL_CALL(CHMreferenceRelease(m_Handle));
   m_Handle = NULL;
   free(m_pDescription);
   free(m_File);
}

CHMstring CHMerror::Dump() const
{
   CHMstring ErrorMessage;
   CHMstring Temp(32,'\0');
   
   sprintf(Temp.get_buffer(),"%d",Code());
   
   ErrorMessage += "Error Code : ";
   ErrorMessage += Temp;
   ErrorMessage += '\n';
   ErrorMessage += "Description : ";
   ErrorMessage += Description();
   ErrorMessage += '\n';

   return ErrorMessage;
}

const long CHMerror::Code() const
{
   return m_Code;
}

CHMchar* CHMerror::Description() const
{
   return m_pDescription;
}

const char* CHMerror::Parameter(const CHMchar* pKey) const
{
   const char* pValue;
   CHM_DLL_CALL(CHMerrorGetParameter(m_Handle, pKey, &pValue));
   return pValue;
}

unsigned int CHMerror::CountOfParameter() const
{
   size_t Value;
   CHM_DLL_CALL(CHMerrorGetCountOfParameter(m_Handle, &Value));
   return Value;
};

CHMboolean CHMerror::HasParameter(const CHMchar* pKey) const
{
   CHMboolean Result;
   CHM_DLL_CALL(CHMerrorHasParameter(m_Handle, pKey, &Result));
   return Result;
};

const char* CHMerror::ParameterName(unsigned int Index) const
{
   const CHMchar* pValue;
   CHM_DLL_CALL(CHMerrorGetKey(m_Handle, Index, &pValue));
   return pValue;
};

#endif // if ! defined ( _USE_MFC_EXCEPTIONS )

void CHMactivateCondition
(
   const CHMchar*    Message,
   long        Line,
   const char* File
)
{
   throw CHMerror(Message, Line, File, 0);
}

void CHMactivateCondition
(
   const CHMchar*    Message,
   long        Line,
   const char* File,
   CHMresult   Code
)
{
#ifdef _USE_MFC_EXCEPTIONS
   throw new CHMerror(Code,Line,File);
#else
   throw CHMerror(Code, Line, File);
 #endif // _MSC_VER
}
