//------------------------------------------------------------------------
//
// Copyright (C) 1997-2004 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module:  CHMstring
// 
// Description
//
// String class derived from work by Lynn Prentice.
//
// Author: Eliot Muir
// Date:   Wed 09/09/1998 
// Revision: $Revision: 1.21 $
//
// Last Edit Date: $Date: 2009-01-14 05:20:24 $
// Source: $Source: /home/cvs/cvsroot/CHM/CHMstring.cpp,v $
//---------------------------------------------------------------------------

#include <CHM/CHMpreCompHead.h>
#pragma hdrstop

#ifdef CHM_NO_PRECOMPILED_HEADERS
#   include <CHM/CHMstring.h>
#   include <CHM/CHMerrorClass.h>
#endif
#ifdef CHM_UNIX
#   include <ctype.h>
#endif

#define COPYONWRITE

// comment on to get paranoid comparision checking
//#define PARANOID_CMP_CHECK

// comment out to get rid of development extra preconds..
//#define STRING_DEVCHECKS

// comment out to remove copy on write
//#ifdef STRING_COPYONWRITE

//#endif

#ifdef PARANOID_CHECK
   #undef COPYONWRITE
   #define STRING_DEVCHECKS
   #define PARANOID_CMP_CHECK
#endif

class CHMstringRef 
{
protected:
   enum FlagSet { deallocate = 1 };

   // Data
   CHMchar*        Buffer;
   size_t       Capacity;
   #ifdef COPYONWRITE
      size_t       CountOfReference;
   #endif

   CHMstringRef(const CHMstringRef&);
   const CHMstringRef& operator =(const CHMstringRef&);

public:
   // Constructors
   CHMstringRef(void);

   // Destructor
   virtual ~CHMstringRef() {}

   void Copy(const CHMstringRef& Reference) ;
   void Copy(const CHMchar* Reference, const size_t& Length) ;
   void Dup(CHMchar Character, size_t Length) ;
   void Join(const CHMchar* Reference1, const size_t& Length1,
           const CHMchar* Reference2, const size_t& Length2) ;
   void Join(const CHMchar* Reference, const size_t& Length) ;
   void Join( const CHMchar Character ) ;

   operator const CHMchar*(void) const ;
//   operator const size_t&(void) const ;

   CHMchar* buffer() { return Buffer; }

   inline const size_t& Length(void) const
   {
      return CountOfCharacter;
   };

   const size_t& capacity(void) const
   {
      return Capacity;
   };
   size_t       CountOfCharacter;


   #ifdef COPYONWRITE
      void operator++(int) ;
      void operator++(int) const ;
      void operator--(int) ;
      void operator--(int) const ;
      const size_t& Refs(void) const ;
   #endif
};

//=============================================================================

inline CHMstringRef::CHMstringRef(void) 
   : Buffer(NULL),
     CountOfCharacter(0),
     Capacity(0),
     #ifdef COPYONWRITE
         CountOfReference(0)
     #endif
{
};

inline void CHMstringRef::Copy(const CHMstringRef& Reference)
{
   CHMPRECONDITION(Capacity >= Reference.CountOfCharacter);
   CHMPRECONDITION(Buffer != NULL);
   CHMPRECONDITION(Reference.Buffer != NULL);
   CHMstrncpy(Buffer, Reference.Buffer, Reference.CountOfCharacter);
   // cautious..
   CountOfCharacter = Reference.CountOfCharacter;
   Buffer[CountOfCharacter] = 0;
};

inline void CHMstringRef::Copy
(
   const CHMchar* Reference, 
   const size_t& Length
) 
{
   CHMPRECONDITION(Capacity >= Length);
   CHMPRECONDITION(Buffer != NULL);
   CHMPRECONDITION(Reference != NULL);
   CHMstrncpy(Buffer, Reference, Length);
   Buffer[Length] = 0;
   // cautious..
   CountOfCharacter = Length;
};

void CHMstringRef::Dup
(
   CHMchar Character, 
   size_t Length
) 
{
   CHMPRECONDITION(Capacity >= Length);
   CHMPRECONDITION(Buffer != NULL);
   ::memset(Buffer, Character, Length);
   Buffer[Length] = 0;

   // cautious..
   CountOfCharacter = Length;
};

void CHMstringRef::Join
(
   const CHMchar* Reference1, 
   const size_t& Length1,
   const CHMchar* Reference2, 
   const size_t& Length2
) 
{
   CHMPRECONDITION(Capacity >= Length1+Length2);
   CHMPRECONDITION(Buffer != NULL);
   CHMstrncpy(&Buffer[0], Reference1, Length1);
   Buffer[Length1] = 0;
   CountOfCharacter = Length1;
   Join(Reference2, Length2);
};

void CHMstringRef::Join
(
   const CHMchar* Reference, 
   const size_t& Length
)
{
   CHMPRECONDITION(Capacity >= CountOfCharacter+Length);
   CHMPRECONDITION(Buffer != NULL);
   CHMstrncpy(&Buffer[CountOfCharacter], Reference, Length);
   CountOfCharacter += Length;
   Buffer[CountOfCharacter] = 0;
};

void CHMstringRef::Join( const CHMchar Character )
{
//   CHMPRECONDITION( capacity >= nchars + 1 );
//   CHMPRECONDITION( array != NULL );
   Buffer[ CountOfCharacter++ ] = Character;
   Buffer[ CountOfCharacter ] = 0;
}

inline CHMstringRef::operator const CHMchar*(void) const
{
   return Buffer;
};

//CHMstringRef::operator const size_t&(void) const
//{
//   return nchars;
//};

#ifdef COPYONWRITE
   inline void CHMstringRef::operator++(int) 
   {
      CountOfReference++;
   };

   inline void CHMstringRef::operator++(int) const 
   {
      // using old C style cast for compatibility with older compilers
      ((size_t&)CountOfReference)++;
   };

   inline void CHMstringRef::operator--(int)
   {
      CHMPRECONDITION(CountOfReference != 0);
      CountOfReference--;
   };

   inline void CHMstringRef::operator--(int) const 
   {
      CHMPRECONDITION(CountOfReference != 0);
      ((size_t&)CountOfReference)--;
   };

   inline const size_t& CHMstringRef::Refs(void) const
   {
      return CountOfReference;
   };
#endif

//=============================================================================

class CHMstringRef16 : public CHMstringRef
{
   CHMchar Block[17];
public:
   CHMstringRef16(void) ;
};

inline CHMstringRef16::CHMstringRef16(void)
{
   Capacity = 16;
   Buffer = Block;
   *Buffer = '\0';
   //::memset(&block, 0, 17);
};

//=============================================================================

class CHMstringRef64 : public CHMstringRef
{
   CHMchar Block[65];
public:
   CHMstringRef64(void) ;
};

inline CHMstringRef64::CHMstringRef64(void) 
{
   Capacity = 64;
   Buffer = Block;
   *Buffer = '\0';
   //::memset(&block, 0, 65);
};

//=============================================================================

class CHMstringRef256 : public CHMstringRef
{
   CHMchar Block[257];
public:
   CHMstringRef256(void) ;
};

inline CHMstringRef256::CHMstringRef256(void) 
{
   Capacity = 256;
   Buffer = Block;
   *Buffer = '\0';
   //::memset(&block, 0, 257);
};

//=============================================================================

class CHMstringRefN : public CHMstringRef
{
public:
   CHMstringRefN(size_t Length) ;
   ~CHMstringRefN() ;
};

inline CHMstringRefN::CHMstringRefN(size_t Length)
{
   Capacity = Length;
   Buffer = new CHMchar[Length+1];
   *Buffer = '\0';
   //::memset(array, 0, len+1);
};

inline CHMstringRefN::~CHMstringRefN()
{
   delete[] Buffer;
}

//=============================================================================

static inline CHMstringRef* Allocate(const size_t& CountOfChars = 0) 
{
   if (CountOfChars <= 16)
   {
      return new CHMstringRef16();
   }
   else if (CountOfChars <= 64)
   {
      return new CHMstringRef64();
   }
   else if (CountOfChars <= 256)
   {
      return new CHMstringRef256();
   }
   return new CHMstringRefN(CountOfChars*2);  // assume big strings require doubling
};

//=============================================================================
// utiities

const size_t CHMlengthWithLimit
(
   const CHMchar* pBuffer, 
   const size_t& Limit
)
{
   size_t CharacterIndex = 0;
   
   while (CharacterIndex < Limit && pBuffer[CharacterIndex] != 0)
   {
      CharacterIndex++;
   }
   
   return CharacterIndex;
};


static CHMstring EmptyString("");

//=============================================================================
// Constructors

CHMstring::CHMstring(const CHMstring& SourceString)
   :   pReference(NULL), Current(NULL)
{
   if (NULL == SourceString.pReference)
   {
      return;
   }
   // attach to the one from the other string
   #ifdef COPYONWRITE
      pReference = SourceString.pReference;
      (*pReference)++;
   #else
      pReference = ::Allocate(SourceString.pReference->Length());
      pReference->Copy(*SourceString.pReference);
   #endif
   Current = *pReference;
};

CHMstring::CHMstring
(
   const CHMstring& SourceString,
   const size_t& StartIndex,
   const size_t& CountOfCharacters
) 
   :pReference(NULL), Current(NULL)
{
   // could speed this up with a clone operation on CHMstringRef
   if (NULL == SourceString.pReference)
   {
      return;
   }

   if ((StartIndex == 0) && 
       (CountOfCharacters >= SourceString.pReference->Length()))      // same as a simple copy - refs..
   {  
      #ifdef COPYONWRITE
         pReference = SourceString.pReference;
      #else      
         pReference = ::Allocate(SourceString.pReference->Length());
         pReference->Copy(*SourceString.pReference);
      #endif
   }
   else if (StartIndex >= SourceString.pReference->Length()) //empty string
   {
      pReference = NULL;
      Current = CHM_TEXT("");
      return;
   }
   else                     // test limits thoroughly...
   {
      size_t SourceStringLength = SourceString.pReference->Length();
      CHMPRECONDITION(StartIndex < SourceStringLength);
      size_t Length = CountOfCharacters;
      if ((CountOfCharacters == npos) || 
          (StartIndex + Length> SourceStringLength))
      {
         Length = SourceStringLength - StartIndex;
      }
//      CHMPRECONDITION(orig + len <= olen);
      pReference = ::Allocate(Length);
      pReference->Copy(&SourceString.c_str()[StartIndex], Length);
   }
   #ifdef COPYONWRITE
      (*pReference)++;
   #endif
   Current = *pReference;
};

CHMstring::CHMstring(const CHMchar* SourceString)
   :   pReference(NULL), Current(NULL)
{
   if(SourceString == NULL)
   {
      return;
   }
   else
   {
      size_t SourceLength = CHMstrlen(SourceString);   // have to  look at the whole string
      if (SourceLength > 0) 
      {
         pReference = ::Allocate(SourceLength);
         pReference->Copy(SourceString, SourceLength);
      }
      else
      {
         return;
      }
   }
   #ifdef COPYONWRITE
      (*pReference)++;
   #endif
   Current = *pReference;
};

CHMstring::CHMstring
(
   const CHMchar* SourceString,
   const size_t& StartIndex,
   const size_t& CountOfCharacters
)
   :   pReference(NULL), Current(NULL)
{
   CHMPRECONDITION(NULL != SourceString);
   
   size_t SourceLength;
   if  (CountOfCharacters == npos)
   {
      SourceLength = CHMstrlen(SourceString);
   }
   else
   {
      SourceLength = ::CHMlengthWithLimit(SourceString, 
                                          StartIndex+CountOfCharacters+1);   // prevent walk off the end...
   }
   
   CHMPRECONDITION(StartIndex <= SourceLength);
   size_t NewLength = CountOfCharacters;
   if ((CountOfCharacters == npos) || 
       (StartIndex + NewLength > SourceLength))
   {
      NewLength = SourceLength - StartIndex;
   }
//   CHMPRECONDITION(orig + len <= olen);
   pReference = ::Allocate(NewLength);
   #ifdef COPYONWRITE
      (*pReference)++;
   #endif
   pReference->Copy(&SourceString[StartIndex], NewLength);
   Current = *pReference;
};

CHMstring::CHMstring(CHMchar Character)
   :   pReference(NULL), Current(NULL)
{
   pReference = new CHMstringRef16();
   #ifdef COPYONWRITE
      (*pReference)++;
   #endif
   pReference->Copy(&Character, 1);
   Current = *pReference;
};

CHMstring::CHMstring
(
   size_t InitialSize, 
   CHMchar InitializationCharacter
) 
   :   pReference(NULL), Current(NULL)
{
   pReference = ::Allocate(InitialSize);
   #ifdef COPYONWRITE
      (*pReference)++;
   #endif
   pReference->Dup(InitializationCharacter, InitialSize);
   Current = *pReference;
};

CHMstring& CHMstring::operator =(const CHMstring& SourceString)
{
   if (pReference == SourceString.pReference)      // we are the same...
      return *this;
   
   if (NULL != pReference)
   {
   #ifdef COPYONWRITE
      (*pReference)--;
      if (pReference->Refs() == 0)
         delete pReference;
   #else
      delete pReference;
   #endif
      pReference = NULL;
   }
   //current = NULL; // unneeded

   // allocate to the new
   #ifdef COPYONWRITE
      pReference = SourceString.pReference;
      if (NULL != pReference)
      {
         (*pReference)++;
      }
   #else
      if (SourceString.pReference != NULL)
      {
         pReference = ::Allocate(s.pReference->Length());
         pReference->Copy(*SourceString.pReference);
      }
      else
      {
         pReference = NULL;
      }
      CHMPOSTCONDITION(pReference == NULL);
   #endif  // COPYONWRITE

   if (pReference != NULL)
   {
      Current = *pReference;
   }
   else
   {  
      Current = NULL;
   }
   CHMPRECONDITION((Current == NULL && pReference == NULL) ||
                    (Current == *pReference));
   return *this;
};

CHMstring& CHMstring::operator += (const CHMstring& SourceString)
{
   operator +=(SourceString.c_str());
   return *this;
};

CHMstring& CHMstring::operator += (const CHMchar* SourceString)
{
   // this is a 'write' operation
   CHMPRECONDITION(SourceString != NULL);
   return append(CHMstrlen(SourceString),SourceString);
};

// Destructor
CHMstring::~CHMstring(void) 
{
   #ifdef COPYONWRITE
      if (pReference != NULL)
      {
         CHMPRECONDITION(pReference != NULL);
         (*pReference)--;
         if (pReference->Refs() == 0)
            delete pReference;
      }
   #else
       delete pReference;  // NULL pointers can be deleted
   #endif
};

//=============================================================================
// heavily used methods...

static const size_t Zero = 0;

const size_t& CHMstring::size(void) const
{
   // This might not be that efficient...
   if (!pReference)
   {
      return Zero;
   }
   //CHMPRECONDITION(pReference != NULL);
   return pReference->Length();
};

//-----------------
// find

const size_t CHMstring::find
(
   const CHMstring& Pattern, 
   const size_t& StartIndex
) const 
{
   if (Pattern.is_null())
   {
      return npos;
   }
   return find_index(Pattern.c_str(), StartIndex);
};

const size_t CHMstring::find_index
(
   const CHMchar* Pattern,
   const size_t& StartIndex
) const
{
   const char* pStart = c_str();
   const char* pStringStart = pStart;
   const char* pFound = NULL;
  
   pStart += StartIndex;

   if (!(pFound = strstr( pStart, Pattern)))
   {
      return npos;
   }
   else
   {
      return pFound-pStringStart;
   }
};

const size_t CHMstring::rfind
(
   const CHMstring& Pattern, 
   const size_t& StartIndex
) const 
{
   if (Pattern.is_null())
   {
      return npos;
   }
   return rfind_index(Pattern.c_str(), StartIndex);
};

const size_t CHMstring::rfind_index
(
   const CHMchar* Pattern,
   const size_t& StartIndex
) const
{
   if (NULL == pReference)
   {
      return npos;
   }

   size_t PatternLength = CHMstrlen(Pattern);
   int Position = StartIndex;
   size_t Size = size();

   if ((size_t) Position >= Size)
   {
      Position = Size - 1;
   }

   const char* cstring = Current + Position;

   while (Position >= 0)
   {
      if (CHMstrlen(cstring) >= PatternLength)
      {
         for (size_t CharIndex = 0; CharIndex < PatternLength; CharIndex++)
         {
            if (cstring[CharIndex] != Pattern[CharIndex])
            {
               break;
            }
            else if (CharIndex == PatternLength - 1)
            {  
               return Position;
            }
         }
      }
      Position--;
      cstring = Current + Position;
   }

   return npos;          // Not found.
};

void CHMstring::assign(size_t Count, CHMchar Filler)
{
   operator=(CHMstring(Count,Filler));
}


void CHMstring::stripAll(const CHMchar& StripCharacter)
{
   CHMchar* pBuffer = get_buffer();
   size_t End = size();
   if (End == 0)
   {
      return;  // we have an empty string
   }
   
   size_t CharIndex =0;

   for (size_t CopyIndex = 0; CopyIndex < End; CopyIndex++)
   {
      if (pBuffer[CopyIndex] != StripCharacter)
      {
         pBuffer[CharIndex] = pBuffer[CopyIndex];
         CharIndex++;
      }
   }
   pBuffer[CharIndex] = '\0';
 
   pReference->CountOfCharacter = CHMstrlen(pBuffer);
}

void CHMstring::subChar
(
   const CHMchar pTarget,
   const CHMchar pReplacement
)
{
   CHMchar* pBuffer = get_buffer();
   size_t End = size();

   for (size_t CharIndex = 0;
        CharIndex < End;
        CharIndex++)
   {
      if (pBuffer[CharIndex] == pTarget)
      {
         pBuffer[CharIndex] = pReplacement;
      }
   }
}

CHMstring CHMstring::strip
(
   const CHMstring::StripType& Type, 
   const CHMchar& StripCharacter
) const
{
   if (size() == 0)
   {
      return CHMstring();  // null string
   }
   size_t StartIndex = 0;                        // index of first character
   size_t EndIndex = size() - 1;                 // index of last character
   // have to use slow size() method above since string could be null.
   const CHMchar* Direct = c_str();

   // check for start and special case of resulting empty string
   if ((Type == Leading) || 
       (Type == Both))
   {
      for(; StartIndex <= EndIndex; StartIndex++)
      {
         if (Direct[StartIndex] != StripCharacter)
         {
            goto nonNull;
         }
      }
      return CHMstring(); // Null string
   }

nonNull:
   if ((Type == Trailing) || 
       (Type == Both))
   {
      for(; EndIndex >= StartIndex; EndIndex--)
         if( Direct[EndIndex] != StripCharacter )
         {
            break;
         }
   }
   return CHMstring(*this, StartIndex, EndIndex - StartIndex + 1);
};

int CHMstring::compare(const CHMstring& ComparisonString) const 
{
   // fast return if we have the same string...
   if (pReference == ComparisonString.pReference)
      return 0;
   return CHMstrcmp(c_str(), ComparisonString.c_str());
};

int CHMstring::compare(const CHMchar* ComparisonString) const
{
   CHMPRECONDITION(NULL != ComparisonString);
   return CHMstrcmp(c_str(), ComparisonString);
};

CHMstring CHMstring::substr
(
   const size_t& StartIndex, 
   const size_t& CountOfCharacters
) const 
{
   return CHMstring(*this, StartIndex, CountOfCharacters);
};

CHMstring& CHMstring::replace
( 
   const size_t& StartIndex,
   const size_t& CountOfCharacters,
   const CHMstring& Replacement,
   const size_t& ReplacementStartIndex,
   const size_t& ReplacementCountOfCharacters
) 
{
   if (NULL == pReference)
   {
      return *this;
   }
   CHMPRECONDITION(pReference != NULL);
   if( StartIndex > pReference->Length() )
   {
      return *this;
   }

   {
      // get the first section
      CHMstring TemporayString(*this, 0, StartIndex);
      
      // get the replace section
      if (ReplacementStartIndex < Replacement.size())
      {
         TemporayString.append(Replacement.c_str(),
                               ReplacementStartIndex,
                               ReplacementCountOfCharacters);
      }
      
      // if anything left over then add that
      if ((CountOfCharacters != npos) && 
          (CountOfCharacters < (size() - StartIndex)))
      {
         TemporayString.append(c_str(), StartIndex + CountOfCharacters, npos);
      }

      // now shuffle the internals..
      #ifdef COPYONWRITE
         (*pReference)--;
         if (pReference->Refs() == 0)
         {
            delete pReference;
         }
      #else
         delete pReference;
      #endif
      pReference = TemporayString.pReference;
      #ifdef COPYONWRITE
         (*pReference)++;
      #else
         TemporayString.pReference = NULL;      // prevent temp delete
      #endif
   }
   Current = *pReference;
   return *this;
};

CHMstring& CHMstring::remove
(
   const size_t& StartIndex, 
   const size_t& CountOfCharacters
)
{
   if (NULL == pReference)
   {
      return *this;
   }
   CHMPRECONDITION(pReference != NULL);
   if (StartIndex > pReference->Length())
   {
      return *this;
   }
   
   {
      // first section
      CHMstring TemporaryString(*this, 0, StartIndex);
      
      // remainder
      if ((CountOfCharacters != npos) && 
          (CountOfCharacters < (pReference->Length() - StartIndex)))
      {
         TemporaryString.append(c_str(),StartIndex + CountOfCharacters,npos);
      }

      // now shuffle the internals..
      #ifdef COPYONWRITE
         (*pReference)--;
         if (pReference->Refs() == 0)
         {
            delete pReference;
         }
      #else
         delete pReference;
      #endif

      pReference = TemporaryString.pReference;
      #ifdef COPYONWRITE
         (*pReference)++;
      #else
         TemporaryString.pReference = NULL;
      #endif
   }
   Current = *pReference;
   return *this;
};

CHMstring& CHMstring::append
(
   const CHMchar* SourceString, 
   const size_t& StartIndex, 
   const size_t& CountOfCharacters
)
{
   if (SourceString != NULL)
   {      
      if (CountOfCharacters == npos)
      {
         // do a straight operator+= append of the CHMchar*
         operator+=(SourceString + StartIndex);
      }
      else
      {
         // construct CHMchar buffer and then append it using operator+=
         CHMchar* pTempBuffer = new CHMchar[CountOfCharacters + 1];
         CHMPRECONDITION(pTempBuffer != NULL)
         CHMstrncpy(pTempBuffer, SourceString + StartIndex,CountOfCharacters);
         pTempBuffer[CountOfCharacters] = 0; // NULL terminate
         operator +=(pTempBuffer);
         delete [] pTempBuffer; // deallocate the temp buffer
      }
   }
   return *this;
};

CHMstring& CHMstring::prepend
(
   const CHMchar* SourceString,
   const size_t& StartIndex,
   const size_t& CountOfCharacters
)
{
   if (NULL == pReference)
   {
      // just a copy
      operator=(CHMstring(SourceString, StartIndex, CountOfCharacters));
      return *this;
   }

   CHMPRECONDITION(pReference != NULL);
   if (SourceString != NULL)
   {
      // we will use a rougher method.. seldom ever used..
      CHMstring TemporaryString(SourceString, StartIndex, CountOfCharacters);
      TemporaryString += *this;

      // now shuffle the internals..
      #ifdef COPYONWRITE
         (*pReference)--;
         if (pReference->Refs() == 0)
         {
            delete pReference;
         }
      #else
         delete pReference;
      #endif

      pReference = TemporaryString.pReference;
      #ifdef COPYONWRITE
         (*pReference)++;
      #else
         TemporaryString.pReference = NULL;
      #endif
      Current = *pReference;
   }
   return *this;
};

CHMstring& CHMstring::insert
(
   size_t StartIndex,
   const CHMchar* SourceString,
   const size_t& SourceStartIndex,
   const size_t& SourceCountOfCharacters
) 
{
   if (NULL == pReference)
   {
      // just a copy
      operator=(CHMstring(SourceString, SourceStartIndex, SourceCountOfCharacters));
      return *this;
   }

   CHMPRECONDITION(pReference != NULL);
   if (SourceString != NULL)
   {
      // we will use a rougher method.. seldom ever used..
      CHMstring TemporaryString(*this, 0, StartIndex);
      TemporaryString.append(SourceString, SourceStartIndex, SourceCountOfCharacters);
      TemporaryString.append(c_str(), StartIndex, npos);

      // now shuffle the internals..
      #ifdef COPYONWRITE
         (*pReference)--;
         if (pReference->Refs() == 0)
         {
            delete pReference;
         }
      #else
         delete pReference;
      #endif

      pReference = TemporaryString.pReference;
      #ifdef COPYONWRITE
         (*pReference)++;
      #else
         TemporaryString.pReference = NULL;
      #endif
      Current = *pReference;
   }
   return *this;
};

CHMchar* CHMstring::get_buffer()  // TODO: should just be buffer()
{
   if (pReference == NULL)
   {
      return (CHMchar*)CHM_TEXT("");
   }

   // don't use this method for an empty string!
   #ifdef COPYONWRITE
   if (pReference->Refs() > 1)      // copy on write...
   {
      CHMstringRef* pp = ::Allocate(pReference->Length());
      pp->Copy(*pReference);
      (*pReference)--;
      if (pReference->Refs() == 0)
      {
         delete pReference;
      }
      pReference = pp;
      (*pReference)++;
      Current = *pReference;
   }
   #endif
   CHMPRECONDITION(pReference->buffer() != NULL);
   return pReference->buffer();
}

CHMchar& CHMstring::operator[](const size_t& Index) 
{
   CHMPRECONDITION(Index < CHMstring::size());
   CHMPRECONDITION(pReference != NULL);
   #ifdef COPYONWRITE
   if (pReference->Refs() > 1)      // copy on write...
   {
      CHMstringRef* pp = ::Allocate(pReference->Length());
      pp->Copy(*pReference);
      (*pReference)--;
      if (pReference->Refs() == 0)
         delete pReference;
      pReference = pp;
      (*pReference)++;
      Current = *pReference;
   }
   #endif
   // using C style cast for compatibility with older compilers
   return (CHMchar&)((c_str())[Index]);
};

CHMstring operator +(const CHMchar* String1, const CHMstring& String2) 
{
   CHMstring Temp(String1);
   Temp += String2;
   return Temp;
};

CHMstring operator +(const CHMstring& String1, const CHMchar* String2) 
{
   CHMstring Temp(String1);
   Temp += String2;
   return Temp;
};

CHMstring operator +(const CHMchar& Character, const CHMstring& String)
{
   return CHMstring(Character) + String;
};

CHMstring operator +(const CHMstring& String, const CHMchar& Character)
{
   return String + CHMstring(Character);
};

#ifdef __BORLANDC__
   CHMstring operator +(const CHMchar& Character, const string& String)
   {
      return CHMstring(Character + String);
   };

   CHMstring operator +(const string& String, const CHMchar& Character)
   {
      return CHMstring(String + Character);
   };
#endif

void CHMstring::clear()
{
   if (NULL != pReference)
   {
   #ifdef COPYONWRITE
      (*pReference)--;
      if (pReference->Refs() == 0)
      {
         delete pReference;
      }
   #else
      delete pReference;
   #endif
      pReference = NULL;
      Current    = NULL;
   }
   CHMPRECONDITION(Current == NULL);
}

CHMstring& CHMstring::append
(
   size_t SizeOfBuffer, 
   const CHMchar* Buffer
)
{
   // this is a 'write' operation
   CHMPRECONDITION(Buffer != NULL);

   // If this only ref and there is enough size we can be economical...   
   if ((pReference) && (pReference->Refs() == 1) &&
       (pReference->capacity() > (pReference->Length() + SizeOfBuffer)))
   {
      // only reference and the capacity is right.
      pReference->Join(Buffer, SizeOfBuffer); 
      return *this;
   }

   // otherwise create a temp of the correct size...
   CHMstringRef* pp = ::Allocate(size() + SizeOfBuffer);   // using size since this could be null
   try
   {
      pp->Join(c_str(), size(), Buffer, SizeOfBuffer);  // using size() since this could be null

      // kill the existing ref
      if (pReference != NULL)
      {
      #ifdef COPYONWRITE
         (*pReference)--;
         if (pReference->Refs() == 0)
         {
            delete pReference;
         }   
      #else
         delete pReference;
      #endif
      }

      // allocate to temp
      pReference = pp;
      #ifdef COPYONWRITE
         (*pReference)++;
      #endif
   }
   catch (...)
   {
      delete pp;
      throw;
   }
   CHMPRECONDITION(pReference != NULL);
   Current = *pReference;
   return *this;
};
