
#include <CHM/CHMpreCompHead.h>
#pragma hdrstop
#ifdef CHM_NO_PRECOMPILED_HEADERS
#  include <CHM/CHMtableClass.h>
#  include <CHM/CHMdateTimeClass.h>
#  include <CHM/CHMerrorClass.h>
#  include <CHM/CHMtable.h>
#  include <CHM/CHMtableRow.h>
#  include <CMH/CHMtableGrammar.h>
#  include <memory.h>
#endif

class CHMtablePrivate
{
public:
   CHMtablePrivate(CHMtableHandle Handle);
   ~CHMtablePrivate();

   void initSubTables();
   void deleteSubTables();

   size_t          CountOfSubTable;
   CHMtable**      ppTableList;
   CHMtableHandle  TableHandle;

   CHMtableRow*    TableRow;
};

CHMtablePrivate::CHMtablePrivate
(
   CHMtableHandle Handle
) : ppTableList(NULL), TableRow(NULL)
{
   if (Handle != NULL)
   {
      TableHandle = Handle;
   }
   else
   {
      CHM_CHECK_CALL(CHMtableCreate(&TableHandle));
   }
   CHM_CHECK_CALL(CHMtableAddRef(TableHandle));
}

CHMtablePrivate::~CHMtablePrivate()
{
   deleteSubTables();
   delete TableRow;
   CHM_CHECK_CALL(CHMtableRelease(TableHandle));
}

// a simple implementation for now - could be made more
// efficient later - the important thing is that we achieve
// a nice interface NOW
CHMtableRow* CHMtable::operator[](size_t RowIndex)
{
   delete pMember->TableRow;
   pMember->TableRow = NULL;
   pMember->TableRow = new CHMtableRow(this, RowIndex);
   return pMember->TableRow;
}

const CHMtableRow* CHMtable::operator[](size_t RowIndex) const
{
   delete pMember->TableRow;
   pMember->TableRow = NULL;
   ((CHMtable*)this)->pMember->TableRow = new CHMtableRow(this, RowIndex);
   return pMember->TableRow;
}

void CHMtablePrivate::initSubTables()
{
   if (ppTableList != NULL) return;

   CHM_CHECK_CALL(CHMtableCountOfSubTable(TableHandle, &CountOfSubTable));
   size_t CountOfRow;
   CHM_CHECK_CALL(CHMtableCountOfRow(TableHandle, &CountOfRow));
   ppTableList = new CHMtable*[CountOfRow*CountOfSubTable];
   // clear the table list pointers so we can avoid an access
   // violation from deleting an invalid pointer.
   memset(ppTableList, 0, CountOfRow * CountOfSubTable * sizeof(CHMtable*));
   for (size_t TableIndex = 0;
        TableIndex < CountOfSubTable;
        ++TableIndex)
   {
      for (size_t RowIndex = 0;
           RowIndex < CountOfRow;
           ++RowIndex)
      {
         CHMtableHandle SubTableHandle;
         CHM_CHECK_CALL(CHMtableSubTable(TableHandle, RowIndex, TableIndex, &SubTableHandle));
         ppTableList[RowIndex * CountOfSubTable + TableIndex] = new CHMtable(SubTableHandle);
      }
   }
}

void CHMtablePrivate::deleteSubTables()
{
   size_t CountOfRow;
   CHM_CHECK_CALL(CHMtableCountOfRow(TableHandle, &CountOfRow));

   if (ppTableList != NULL)
   {
      for (size_t TableIndex = 0;
           TableIndex < CountOfSubTable;
           TableIndex++)
      {
         for (size_t RowIndex = 0; 
              RowIndex < CountOfRow; 
              RowIndex++)
         {
            delete ppTableList[RowIndex * CountOfSubTable + TableIndex];
         }
      }
      delete [] ppTableList;
      ppTableList = NULL;
   }
}

CHMtable::CHMtable
(
   CHMtableHandle Handle
) : pMember(NULL)
{  
   pMember = new CHMtablePrivate(Handle);
}

CHMtable::CHMtable() : pMember(NULL)
{
   pMember = new CHMtablePrivate(NULL);
}

CHMtable::~CHMtable()
{
   delete pMember;
}

size_t CHMtable::CountOfRow() const
{
   size_t Count;
   CHM_CHECK_CALL(CHMtableCountOfRow(TableHandle(), &Count));
   return Count;
}

size_t CHMtable::CountOfColumn() const
{
   size_t Count;
   CHM_CHECK_CALL(CHMtableCountOfColumn(TableHandle(), &Count));
   return Count;
}

const char* CHMtable::TableName() const
{
   char* pName;
   CHM_CHECK_CALL(CHMtableName(TableHandle(), &pName));
   return pName;
}

size_t CHMtable::ColumnIndex(const char* pColumnName) const
{
   size_t ColumnIndex;
   CHM_CHECK_CALL(CHMtableColumnIndex(TableHandle(), pColumnName, &ColumnIndex));
   return ColumnIndex;
}

const char* CHMtable::ColumnName(size_t ColumnIndex) const
{
   const char* pColumnName;
   CHM_CHECK_CALL(CHMtableColumnName(TableHandle(), ColumnIndex, &pColumnName));
   return pColumnName;
}

void CHMtable::AddRow()
{
   pMember->deleteSubTables();
   CHM_CHECK_CALL(CHMtableAddRow(TableHandle()));
}

size_t CHMtable::CountOfSubTable() const
{
   pMember->initSubTables();
   return pMember->CountOfSubTable;
}

CHMtable* CHMtable::SubTable(size_t RowIndex, size_t SubTableIndex) const
{
   ((CHMtable*)this)->pMember->initSubTables();
   CHM_ASSERT(SubTableIndex < pMember->CountOfSubTable);
   CHM_ASSERT(pMember->ppTableList[RowIndex * pMember->CountOfSubTable + SubTableIndex] != NULL);
   return pMember->ppTableList[RowIndex * pMember->CountOfSubTable + SubTableIndex];
}

CHMtableHandle CHMtable::TableHandle() const
{
   return pMember->TableHandle; 
}

CHMdataType CHMtable::ColumnType(size_t ColumnIndex) const
{
   CHMdataType DataType;
   CHM_CHECK_CALL(CHMtableColumnType(TableHandle(), ColumnIndex, &DataType));
   return DataType;
}

void CHMtable::SetString
(
   size_t         RowIndex, 
   size_t         ColumnIndex, 
   const CHMchar* pValue
)
{
   CHM_CHECK_CALL(CHMtableSetString(TableHandle(), ColumnIndex, RowIndex, pValue));
}

void CHMtable::SetDouble
(
   size_t RowIndex, 
   size_t ColumnIndex, 
   double Value
)
{
   CHM_CHECK_CALL(CHMtableSetDouble(TableHandle(), ColumnIndex, RowIndex, Value));
}

void CHMtable::SetInteger
(
   size_t RowIndex,
   size_t ColumnIndex, 
   long   Value
)
{
   CHM_CHECK_CALL(CHMtableSetInteger(TableHandle(), ColumnIndex, RowIndex, Value));
}

void CHMtable::SetDateTime
(
   size_t             RowIndex, 
   size_t             ColumnIndex, 
   const CHMdateTime& Value
)
{
   CHM_CHECK_CALL(CHMtableSetDateTime(TableHandle(), ColumnIndex, RowIndex, Value.DateTimeHandle()));
}

const CHMchar* CHMtable::GetString  
(
   size_t RowIndex, 
   size_t ColumnIndex
) const
{
   const CHMchar* Value;
   CHM_CHECK_CALL(CHMtableGetString(TableHandle(), ColumnIndex, RowIndex, &Value));
   return Value;
}

double CHMtable::GetDouble  
(
   size_t RowIndex, 
   size_t ColumnIndex
) const
{
   double Value;
   CHM_CHECK_CALL(CHMtableGetDouble(TableHandle(), ColumnIndex, RowIndex, &Value));
   return Value;
}

long CHMtable::GetInteger 
(
   size_t RowIndex, 
   size_t ColumnIndex
) const
{
   long Value;
   CHM_CHECK_CALL(CHMtableGetInteger(TableHandle(), ColumnIndex, RowIndex, &Value));
   return Value;
}

const CHMdateTime CHMtable::GetDateTime
(
   size_t RowIndex, 
   size_t ColumnIndex
) const
{
   CHMdateTimeHandle Handle;
   CHM_CHECK_CALL(CHMtableGetDateTime(TableHandle(), ColumnIndex, RowIndex, &Handle));
   return CHMdateTime(Handle);
}

CHMtable::State CHMtable::GetState(size_t RowIndex, size_t ColumnIndex) const
{
   size_t StateInt = 0;
   CHM_CHECK_CALL(CHMtableState(TableHandle(), ColumnIndex, RowIndex, &StateInt));
   return (State)StateInt;
}

CHMboolean CHMtable::IsNull
(
   size_t RowIndex, 
   size_t ColumnIndex
) const
{
   CHMboolean IsNull;   
   CHM_CHECK_CALL(CHMtableIsNull(TableHandle(), ColumnIndex, RowIndex, &IsNull));
   return IsNull;
}

void CHMtable::setNull
(
   size_t RowIndex, 
   size_t ColumnIndex
) const
{
   CHM_CHECK_CALL(CHMtableSetNull(TableHandle(), ColumnIndex, RowIndex));
}

CHMboolean CHMtable::IsKey
(
   size_t ColumnIndex
) const
{
   size_t IsKey;
   CHM_CHECK_CALL(CHMtableGetColumnIsKey(TableHandle(), ColumnIndex, &IsKey));
   return IsKey!=0;
}

CHMboolean CHMtable::IsNode() const
{
   CHMtableGrammarHandle TableGrammar = NULL;
   CHM_CHECK_CALL(CHMtableGetTableGrammar(TableHandle(), &TableGrammar));
   CHM_ASSERT( TableGrammar != NULL );
   size_t pFlag = 0;
   CHM_CHECK_CALL(CHMtableGrammarGetIsNode(TableGrammar, &pFlag));
   return pFlag != 0;
}

CHMstring CHMtable::Dump() const
{
   CHMstring Dump;
   CHMchar* pBuffer;
   
   CHM_CHECK_CALL(CHMtableDumpT(TableHandle(), &pBuffer));
   Dump = pBuffer;
   CHM_CHECK_CALL(CHMtableFreeBufferT(&pBuffer));

   return Dump;
}
