//#include <CHM/CHMtableRow.h>
//#include <CHM/CHMerrorClass.h>
//#include <CHM/CHMtableClass.h>

//#include <CHM/CHMtable.h>
//#include <memory.h>
#include <CHM/CHMpreCompHead.h>
#pragma hdrstop

#ifdef CHM_NO_PRECOMPILED_HEADERS
#  include <CHM/CHMtableRow.h>
#  include <CHM/CHMtableClass.h>
#  include <CHM/CHMerrorClass.h>
#endif

class CHMtableRowPrivate //: public CHMinstanceHeap
{
public:
   CHMtableRowPrivate(CHMtable* pTable, size_t RowIndex);
   ~CHMtableRowPrivate() {}

   size_t          RowIndex;
   CHMtable*       pTable;
};

CHMtableRowPrivate::CHMtableRowPrivate
(
   CHMtable* ipTable,
   size_t    iRowIndex
) 
{
   CHM_ASSERT(ipTable != NULL);
   pTable = ipTable;
   RowIndex = iRowIndex;
}

CHMtableRow::CHMtableRow
(
   const CHMtable* pTable,
   size_t    RowIndex
) : pMember(NULL)
{  
   CHM_ASSERT_COMMENT(RowIndex < pTable->CountOfRow(), "Boundary error with row index for table.  This most commonly occurs when users write code which attempts to access the first row of an empty table.  Robust code should check that the Count of rows is at least 1 before attempting to access data in the table.  See the FAQ on the website.");
   pMember = new CHMtableRowPrivate((CHMtable*)pTable, RowIndex);
}

CHMtableRow::~CHMtableRow()
{
   delete pMember;
}

size_t CHMtableRow::RowIndex() const
{
   return pMember->RowIndex;
}

CHMtableHandle CHMtableRow::TableHandle() const
{
   return pMember->pTable->TableHandle(); 
}

const CHMtable* CHMtableRow::SubTable(size_t SubTableIndex) const
{
   CHM_ASSERT(pMember->pTable != NULL);
   CHM_ASSERT(SubTableIndex < pMember->pTable->CountOfSubTable());
   return pMember->pTable->SubTable(pMember->RowIndex, SubTableIndex);
}

CHMtable* CHMtableRow::SubTable(size_t SubTableIndex)
{
   CHM_ASSERT(pMember->pTable != NULL);
   CHM_ASSERT(SubTableIndex < pMember->pTable->CountOfSubTable());
   return pMember->pTable->SubTable(pMember->RowIndex, SubTableIndex);
}
