unit CHMinterface;

interface

uses Sysutils;

type TLongIntPtr = ^LongInt;
type TCardinalPtr = ^Cardinal;

{$IFDEF VER100}
type Longword = Longint;
type HResult = Longint;
{$ENDIF}

type TLongWordPtr = ^Longword;
type TDoublePtr = ^Double;
type TShortIntPtr = ^ShortInt;
type TCHMresultHandle = Pointer;
type TDateTimePtr = ^TDateTime;
type TBooleanPtr = ^Boolean;
type TIntegerPtr = ^Integer;
type TPCharPtr = ^PChar;

type
   TCHMiunknown = class(TObject)
   protected
      FReferenceCount : Integer;
   public
      constructor Create;
      function _AddRef: Integer; virtual; stdcall;
      function _Release: Integer; virtual; stdcall;

      function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
   end;

type
   TCHMexception = class(Exception)
   private
       FLongDescription : string;
       function Get_Message : string;
   public
       function LongDescription : string;
       constructor CreateLong(Description : string);
       property Message : string read Get_Message ;
   end;

procedure DLLcheckResult(ErrorHandle : TCHMresultHandle);

type TCHMexceptionHandle = Pointer;
type TCHMexceptionHandlePtr = ^TCHMexceptionHandle;

type ICHMexception = interface
   {$IFNDEF HIDE}
   {$ENDIF}

   {:
     A text message describing the error.
   }
   function GetDescription : String;

   {:
     An error code representing the error
   }
   function GetCode : Integer;

   {:
     Errors may contain additional information based on name value pairs. This allows
     retrieval of that extra information by providing a key.
   }
   function GetParameter(Key : String) : String;

   {:
     Number of extra parameters that represent additional error information.
   }
   function GetCountOfParameter : Integer;

   {:
     Returns the key name at the given index position.
   }
   function GetKey(KeyIndex : Integer) : String;
   end;

function CHMexceptionCreateObject : ICHMexception;
function CHMexceptionCreateObjectWithHandle(Handle : TCHMexceptionHandle) : ICHMexception;

implementation

uses CHMnet;

type TCHMexceptionImp = class(TCHMiunknown, ICHMexception)
   private
      FHandle : TCHMexceptionHandle;
   public
      constructor Create;
      constructor CreateWithHandle(Handle : TCHMexceptionHandle);
      function _AddRef : Integer; override; stdcall;
      function _Release : Integer; override; stdcall;
      destructor Destroy; override;
      {$IFNDEF HIDE}
      {$ENDIF}

      {:
        A text message describing the error.
      }
      function GetDescription : String;

      {:
        An error code representing the error
      }
      function GetCode : Integer;

      {:
        Errors may contain additional information based on name value pairs. This allows
        retrieval of that extra information by providing a key.
      }
      function GetParameter(Key : String) : String;

      {:
        Number of extra parameters that represent additional error information.
      }
      function GetCountOfParameter : Integer;

      {:
        Returns the key name at the given index position.
      }
      function GetKey(KeyIndex : Integer) : String;
   end;

function CHMexceptionCreate
(
   ReturnValue : TCHMexceptionHandlePtr
) : TCHMresultHandle; stdcall;
external 'NETDLL2.dll' name '_CHMexceptionCreate@4';

function CHMexceptionAddRef
(
   Handle : TCHMexceptionHandle
) : TCHMresultHandle; stdcall;
external 'NETDLL2.dll' name '_CHMexceptionAddRef@4';

function CHMexceptionRelease
(
   Handle : TCHMexceptionHandle
) : TCHMresultHandle; stdcall;
external 'NETDLL2.dll' name '_CHMexceptionRelease@4';

function CHMexceptionGetDescription
(
   Handle : TCHMexceptionHandle;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall;
external 'NETDLL2.dll' name '_CHMexceptionGetDescription@8';

function CHMexceptionGetCode
(
   Handle : TCHMexceptionHandle;
   ReturnValue : TIntegerPtr
) : TCHMresultHandle; stdcall;
external 'NETDLL2.dll' name '_CHMexceptionGetCode@8';

function CHMexceptionGetParameter
(
   Handle : TCHMexceptionHandle;
   Key : PChar;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall;
external 'NETDLL2.dll' name '_CHMexceptionGetParameter@12';

function CHMexceptionGetCountOfParameter
(
   Handle : TCHMexceptionHandle;
   ReturnValue : TIntegerPtr
) : TCHMresultHandle; stdcall;
external 'NETDLL2.dll' name '_CHMexceptionGetCountOfParameter@8';

function CHMexceptionGetKey
(
   Handle : TCHMexceptionHandle;
   KeyIndex : Integer;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall;
external 'NETDLL2.dll' name '_CHMexceptionGetKey@12';

constructor TCHMexceptionImp.Create();
begin
   inherited Create;
   FHandle := nil;
   DLLcheckResult(CHMexceptionCreate(Addr(FHandle)));
end;

constructor TCHMexceptionImp.CreateWithHandle(Handle : TCHMexceptionHandle);
begin
   inherited Create;
   FHandle := Handle;
end;

function TCHMexceptionImp._AddRef : Integer;
begin
   Result := inherited _AddRef;
   DLLcheckResult(CHMexceptionAddRef(FHandle));
end;

function TCHMexceptionImp._Release : Integer;
begin
   DLLcheckResult(CHMexceptionRelease(FHandle));
Result := inherited _Release;
end;

destructor TCHMexceptionImp.Destroy;
begin
   inherited Destroy;
end;

function CHMexceptionCreateObject : ICHMexception;
begin
   Result := TCHMexceptionImp.Create();
end;

function CHMexceptionCreateObjectWithHandle(Handle : TCHMexceptionHandle) : ICHMexception;
begin
   Result := TCHMexceptionImp.CreateWithHandle(Handle);
end;

function TCHMexceptionImp.GetDescription : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMexceptionGetDescription(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

function TCHMexceptionImp.GetCode : Integer;
var
   ResultValue : Integer;
begin
   DLLcheckResult(CHMexceptionGetCode(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

function TCHMexceptionImp.GetParameter
(
   Key : String
) : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMexceptionGetParameter(FHandle, PChar(Key), Addr(ResultValue)));
   Result := ResultValue;
end;

function TCHMexceptionImp.GetCountOfParameter : Integer;
var
   ResultValue : Integer;
begin
   DLLcheckResult(CHMexceptionGetCountOfParameter(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

function TCHMexceptionImp.GetKey
(
   KeyIndex : Integer
) : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMexceptionGetKey(FHandle, KeyIndex, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure DLLcheckResult(ErrorHandle : TCHMresultHandle);
var
   Description : string;
   Error : ICHMexception;
begin
   if TCHMresultHandle(0) <> ErrorHandle then
   begin
      if ErrorHandle = TCHMresultHandle(-1) then
      begin
         raise TCHMexception.CreateLong('Call failed.');
      end;
      Error := CHMexceptionCreateObjectWithHandle(ErrorHandle);
      Description := Error.GetDescription;
      raise TCHMexception.CreateLong(Description);
   end;
end;

{ TCHMiunknown }

constructor TCHMiunknown.Create;
begin
   FReferenceCount :=0;
end;

function TCHMiunknown.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
   Result := 0;
end;

function TCHMiunknown._AddRef: Integer;
begin
   Inc(FReferenceCount);
   Result := FReferenceCount;
end;

function TCHMiunknown._Release: Integer;
begin
   Dec(FReferenceCount);
   Result := FReferenceCount;
   if (0 = FReferenceCount) then
   begin
      Self.Free;
   end;
end;


{ TCHMexception }

constructor TCHMexception.CreateLong(Description: string);
var
   ShortDescription : String;
begin
   ShortDescription := Description;
   FLongDescription := Description;
   if Length(ShortDescription) > 512 then
   begin
      SetLength(ShortDescription, 512);
   end;
   inherited Create(ShortDescription);
end;

function TCHMexception.Get_Message: string;
begin
   Result := FLongDescription;
end;

function TCHMexception.LongDescription: string;
begin
   Result := FLongDescription;
end;

end.
