unit ServerFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ScktComp, CHM, PrefixTest, StdCtrls, ExtCtrls, CHMnet;

type
  TServerForm = class(TForm)
    memOutput: TMemo;
    pnlControls: TPanel;
    btnReset: TButton;
    hl7Engine: TPrefixTestengine;
    edtPortNumber: TEdit;
    lblPortNumber: TLabel;
    llpServer: TCHMllpServer;
    ackEngine : TCHMdynamicEngine;
    nackEngine: TCHMdynamicEngine;
    cbConfiguration: TComboBox;
    lblConfiguration: TLabel;
    procedure hl7EngineLog(Sender: TObject; Line: AnsiString);
    procedure btnResetClick(Sender: TObject);
   // Begin message event declarations (comment used as an anchor for STD)
   procedure hl7EngineMsgMessage1(Sender: TObject; Msg: TPrefixTestmsgMessage1;
     TransportIndex: Integer);
   // End message event declarations (comment used as an anchor for STD)


    procedure FormCreate(Sender: TObject);
    procedure llpServerClientConnected(Sender: TObject;
      Client: ICHMllpConnection);
    procedure llpServerMessageReceived(Sender: TObject;
      Client: ICHMllpConnection; Message: AnsiString);
    procedure llpServerClientDisconnected(Sender: TObject;
      Client: ICHMllpConnection);
    procedure llpServerClientErrorReceived(Sender: TObject;
      Client: ICHMllpConnection; ErrorMessage: AnsiString; ErrorCode: Integer);
    procedure llpServerErrorReceived(Sender: TObject; ErrorMessage: AnsiString;
      Code: Integer);
    procedure llpServerDataIgnored(Sender: TObject;
      Client: ICHMllpConnection; Data: AnsiString);
    procedure hl7EngineParseError(Sender: TObject; MessageData: AnsiString;
      TransportIndex: Integer; Description: AnsiString; Code: Cardinal);
    procedure cbConfigurationChange(Sender: TObject);
  private
    procedure ToggleListener;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  ServerForm: TServerForm;

implementation

{$R *.DFM}

// This application is a simple example of using the Chameleon generated objects.  It
// is intended to be a helpful starting point in understanding how to use these objects
// but must be fleshed out in the following areas to provide a full solution for an
// application HL7 interface:
// 1) You will need to work out how your application will handling logging of messages
//    and data, in particular how bad transactions should be logged.
// 2) The code will need to be modified to insert the parsed data into a database whether
//    by using explicit database calls or going through some type of middle tier layer that
//    your application may support.
// 3) Some customers choose to deploy HL7 interfaces as Windows Services.  This can be done
//    easily with Chameleon.  One thing to keep in mind though is that the networking components
//    supplied with Chameleon require a Windows 'message loop'.  Often times Windows Services do
//    not have a message loop - but one can be easily set up by invoking RunMessageLoop on
//    the networking component supplied with Chameleon.  See the manual for running Chameleon
//    within services.
// 4) Definitely take the time to read the manual and go through the tutorials after looking
//    at this example before attempting to deploy a production solution.  We do provide custom
//    onsite training and technical implementation services if you require them.
//
// If you are looking for a much more complete 'out of the box' solution for your HL7 needs it would be
// a good idea to evaluate Iguana - Iguana is powered by Chameleon but
// provides a complete runtime web based infrastructure for your HL7 interfaces.
// So it provides advanced logging and monitoring capability for you
// which makes it a very attractive option for many customers.

procedure TServerForm.FormCreate(Sender: TObject);
var
   ConfigIndex : Integer;
begin
   // The first step in making the generated hl7Engine component useable is to load
   // the message definition or 'vmd' file.  This file contains all the definitions
   // for the messages, how they are identified and how they should be mapped into the
   // generated data objects based on the tables that you define in the Chameleon GUI.

   // hl7Engine is a generated engine which means that it has an interface based
   // on the Messages and Tables you define in your vmd file.  The use of stub code is
   // helpful since it means that the compiler can catch typos or spelling mistakes of fields at compile
   // rather than run time.  However it is not the only way you can use Chameleon.  It is
   // possible to use a non-stubcode interface to the Chameleon parsing engine which we call
   // the 'dynamic interface'.  Please review the manual documentation for more details if you
   // need this functionality - most customers find that the stub code interface works well for
   // most applications.
   hl7Engine.Load(ExtractFilePath(Application.ExeName) + 'test_data.vmd');

   // Set up the current configuration list box.
   for ConfigIndex := 0 to hl7Engine.CountOfConfig-1 do
   begin
      cbConfiguration.Items.Add(hl7Engine.GetConfigName(ConfigIndex));
   end;
   cbConfiguration.ItemIndex := hl7Engine.CurrentConfigIndex;

   // This example has two other Chameleon HL7 engines to help with the generation of
   // HL7 ACKnowledgement and Negative ACKnowledgment messages.  In this case each engine
   // object is not a stubcode generated engine but is a dynamic engine.  For more information
   // please read the Chameleon manual about auto-acknowledgement.
   ackEngine.Load(ExtractFilePath(Application.ExeName) + 'autoack.vmd');
   nackEngine.Load(ExtractFilePath(Application.ExeName) + 'autonack.vmd');
end;


// We start listening by invoking the method ToggleListener
// which looks at the value in the edit control to determine the port to listen on.
// In a real application of Chameleon this would typically
// be set in a configuration file for your application.  Use whatever conventions you commonly use
// with your application.  Keep in mind that if another application is listening on this port
// - i.e. another instance of this application, that a runtime error will be thrown.
procedure TServerForm.ToggleListener;
var
   NewPortNumber, Code : integer;
begin
   if (llpServer.IsListening) then
   begin
      // This will stop us accepting new connections
      llpServer.StopListen;
      llpServer.DisconnectAllClients;
      // This will disconnect existing connections.
      btnReset.Caption := '&Start Listening';
      hl7Engine.WriteToLog('Stopped listening');
   end
   else
   begin
      Val(edtPortNumber.Text, NewPortNumber, Code);
      if Code <> 0 then
      begin
         ShowMessage('Invalid port number.');
         exit;
      end;
      llpServer.StartListen(NewPortNumber);
      btnReset.Caption := '&Stop Listening';
      hl7Engine.WriteToLog('Started listening on port ' + IntToStr(llpServer.Port));
   end;
end;

// Stop or Start the listener
procedure TServerForm.btnResetClick(Sender: TObject);
begin
   ToggleListener;
end;

procedure TServerForm.hl7EngineLog(Sender: TObject; Line: AnsiString);
begin
   // This method is called everytime we invoke the 'WriteToLog' method on the engine object.  This
   // code is only for demonstration purposes.  Memo controls can only take a certain amount
   // of data - 32k at the time this example was written.  As a result we check the length of
   // the memo control and truncate it if it reaches this limit.

   // It should be strongly emphasized that this solution is only for demonstration purposes.
   // A real HL7 interface should have some type of system of logging into log files.  Iguana for
   // instance has a very nice logging system that allows administrators to drill into and browse
   // processed messages and errors.  How you implement this logging is up to your requirements
   // for run time management of your HL7 interface.
   memOutput.Lines.BeginUpdate;

   if (Length(memOutput.Text) > 30000) then
   begin
      memOutput.Text := Line;
      memOutput.Lines.EndUpdate;
   end
   else
   begin
      memOutput.Lines.EndUpdate;
      memOutput.Lines.Add(Line);
   end;
end;

// This is the first event we would expect to get from a client connecting to us.  Notice
// that all the networking events have a 'Client' object that can be used to interact with
// the peer machine that is sending HL7 data.
procedure TServerForm.llpServerClientConnected(Sender: TObject;
  Client: ICHMllpConnection);
begin
   hl7Engine.WriteToLog('Got connection from ' + Client.RemoteIpAddressAsString);
end;

// This event is fired if we have an error on a given TCP/IP connection.  When you
// develop a real application you should log this information for troubleshooting purposes.
procedure TServerForm.llpServerClientErrorReceived(Sender: TObject;
  Client: ICHMllpConnection; ErrorMessage: AnsiString; ErrorCode: Integer);
begin
   hl7Engine.WriteToLog('Got client error from ' + Client.RemoteIpAddressAsString + ' ' + ErrorMessage);
end;

// This event is fired if we have an error related to the listening socket itself rather
// than a specific connection.  In practice this event is rarely invoked.
procedure TServerForm.llpServerErrorReceived(Sender: TObject;
  ErrorMessage: AnsiString; Code: Integer);
begin
   hl7Engine.WriteToLog('Got server error ' + ErrorMessage);
end;

// HL7 data is sent via the Lower Layer Protocol.  Read the manual for more detailed background
// on this protocol.  If you have a badly behaved counter party it is possible they may send data
// that is outside of the LLP start of block and end of block characters - this event will tell
// you when this problem occurs.
procedure TServerForm.llpServerDataIgnored(Sender: TObject;
  Client: ICHMllpConnection; Data: AnsiString);
begin
   hl7Engine.WriteToLog('Ignored data outside of LLP connection from client ' + Client.RemoteIpAddressAsString + 'Data: ' + Data);
end;

// This event is the main entry point for where we get an actual HL7 message off
// 'the wire' from a connected peer.  Notice that we 'manually' invoke the
// ParseMessage method on the hl7Engine object.  This will
// fire one of the generated onMessage events which follow
// after this method or the ParseError method.
//
// Notice that the LLP TCP/IP transport components are not tightly coupled to the
// hl7Engine object.  You can use any kind of transport you like with Chameleon, including
// your own transports or third party components.
//
// Notice that in this application we do not block on events - this is because the
// networking components are asynchronous.  You can write synchronous applications using
// Chameleon but you would need to use another transport mechanism other than the LLP TCP/IP
// components supplied here.
procedure TServerForm.llpServerMessageReceived(Sender: TObject;
  Client: ICHMllpConnection; Message: AnsiString);
var
   ACK : AnsiString;
begin
   // All our networking events from llpServer have a Client object which gives us
   // information about the host we received a message from.
   hl7Engine.WriteToLog('Received message from ' + Client.GetRemoteIpAddressAsString);
   // We use the MakeMessagePrintable function to 'pretty print'
   // the HL7 message into a format that can be easily read in
   // the log files.  This function converts \r characters into the
   // string given in the second argument of MakeMessagePrintable.
   //    i.e. in this case a \r\n sequence which displays nicely in the memo control.
   // The third argument is the maximum length of the message before
   // we truncate it.  This is done because displaying data within a memo
   // control is actually very slow - so displaying a message of 100k in
   // the memo control window can give the appearance of hanging the server.
   // (i.e. a long blocking event)
   hl7Engine.WriteToLog(llpServer.MakeMessagePrintable(Message, #13#10, 2000));

   // Send an ACKnowledgement or Negative ACKnowledgement back to the client
   // after parsing the message
   try
      try
         // We parse the message with Chameleon using ParseMessage.  The second
         // parameter is set to 0 and used to be part of the transport framework for
         // Chameleon but is no longer required.
         hl7Engine.ParseMessage(Message, 0);
         // Generate an ACK using the auto-acknowledgement engine
         ACK := ackEngine.GenerateAck(Message);
      except
         on Exception do
         begin
            // The ACK generation failed, so we send back a NACK
            try
               // we will try and generate the NACK using our nackEngine
               // if that fails then we will have to send a hard coded nack.
               ACK := nackEngine.GenerateAck(Message);
            except
               on Exception do
               begin
                  // our NACK engine failed to parse the message - this should only
                  // really happen when there is real garbage.
                  ACK := 'MSH|^~\\&|ACME|ACME_APP|XYZ|XYZ_APP|||NACK|XXX|P|2.4|\rMSA|AA|XXX|\r';
               end;
            end;
         end;
      end;
   finally
      // We send the response ACK or NACK message to the client via the TransmitMessage method.
      Client.TransmitMessage(ACK);
   end;
end;

// This is the event we would expect when a client disconnects.  Note that sometimes
// if the client is not well behaved we may not receive this event.
procedure TServerForm.llpServerClientDisconnected(Sender: TObject;
  Client: ICHMllpConnection);
begin
   hl7Engine.WriteToLog('Client disconnected from ' + Client.RemoteIpAddressAsString);
end;

// The next set of events are triggered for each of messages that has been
// defined in the message definition file.  In each case there is a generated
// Msg object.  This object has tables and sub tables with properties that you
// can access to obtain the data that has been parsed out of each message.
//
// It must be emphasized that this will be a mirror of the 'table grammar' for
// each message - i.e. the user defined definitions of each table and the fields
// in those tables as defined in the Chameleon GUI.
// Begin message event functions (comment used as an anchor by STD)
procedure TServerForm.hl7EngineMsgMessage1(Sender: TObject;
Msg: TPrefixTestmsgMessage1; TransportIndex: Integer);
begin
   hl7Engine.WriteToLog('Received a Message1 message.');
end;
// End message event functions (comment used as an anchor by STD)



// This event is fired when an parsing error occurs.  Notice we take care
// to convert all the linefeeds appropriately with MakeMessagePrintable.  This
// may not be necessary if you are dealing with an application which is logging into
// a text file.
procedure TServerForm.hl7EngineParseError(Sender: TObject;
  MessageData: AnsiString; TransportIndex: Integer; Description: AnsiString;
  Code: Cardinal);
begin
   // -1 for the third argument means we are not truncating the error message.
   hl7Engine.WriteToLog(llpServer.MakeMessagePrintable(Description, #13#10, -1));
end;

procedure TServerForm.cbConfigurationChange(Sender: TObject);
begin
   hl7Engine.CurrentConfigIndex := cbConfiguration.ItemIndex;
end;

end.
 

