package com.interfaceware.chameleon;

import java.util.*;


/**
 * <p>This class contains functions to change the current encoding used internally 
 * by Chameleon at runtime. Changes made here affect all currently instantiated 
 * Engines and Chameleon network connections.</p>
 * 
 * <p> The locale set here is used to determine how to </p>
 * 
 * <p> 1) Translate .NET and Java unicode strings to and from the internal 
 * representation used in Chameleon for both Engine and network operations. See <a 
 * href="http://www.interfaceware.com/manual/setting_encoding.html">http://www.interfaceware.com/manual/setting_encoding.html</a>
 *  for more details.</p>
 * 
 * <p> 2) Determine how strings are parsed with an Engine.</p>
 * 
 */
public class ChameleonEncoding
{
   private long ChameleonEncodingHandle;
   
   public ChameleonEncoding() throws ChameleonException
   {
      ChameleonEncodingHandle = CHMchameleonEncodingCreate();
      CHMchameleonEncodingAddRef(ChameleonEncodingHandle);
   }
   
   public ChameleonEncoding(long Handle) throws ChameleonException
   {
      ChameleonEncodingHandle = Handle;
      CHMchameleonEncodingAddRef(ChameleonEncodingHandle);
   }
   
   public void release() throws ChameleonException
   {
      if (0 != ChameleonEncodingHandle)
      {
         CHMchameleonEncodingRelease(ChameleonEncodingHandle, 0);
         ChameleonEncodingHandle = 0;
      }
   }
   
   protected void finalize() throws ChameleonException
   {
      release();
   }
   
   static
   {
      System.loadLibrary("chm_java");
   }
   
   
   /**
    * Get handle. This is part of the internal implementation of Chameleon and should 
    * not be called directly.
    */
   private long getHandle()
   {
      return ChameleonEncodingHandle;
   }
   
   /**
    * <p>Get the translation locale.</p>
    * 
    * <p>The initial value is extracted from the default system locale.</p>
    * 
    */
   static public String getCurrentTranslationLocale() throws ChameleonException
   {
      return CHMchameleonEncodingGetCurrentTranslationLocale();
   }
   
   
   /**
    * <p>Set the translation locale. See <a 
    * href="http://www.interfaceware.com/manual/setting_encoding.html">http://www.interfaceware.com/manual/setting_encoding.html</a>
    *  for more details </p>
    * 
    * @param   NewLocale   New Locale Setting
    */
   static public void setCurrentTranslationLocale(String NewLocale) throws ChameleonException
   {
      CHMchameleonEncodingSetCurrentTranslationLocale(NewLocale);
   }
   
   
   /**
    * <p>Get the total number of supported translation locales. </p>
    * 
    */
   static public int countOfTranslationLocale() throws ChameleonException
   {
      return CHMchameleonEncodingCountOfTranslationLocale();
   }
   
   
   /**
    * <p>Get a translation locale at a particular index.</p>
    * 
    * @param   LocaleIndex   Locale Index
    */
   static public String translationLocale(int LocaleIndex) throws ChameleonException
   {
      return CHMchameleonEncodingTranslationLocale(LocaleIndex);
   }
   
   private native long CHMchameleonEncodingCreate() throws ChameleonException;
   private native void CHMchameleonEncodingAddRef(long ChameleonEncodingHandle) throws ChameleonException;
   private native void CHMchameleonEncodingRelease(long ChameleonEncodingHandle, long EventHandle) throws ChameleonException;
   static 
   private native String CHMchameleonEncodingGetCurrentTranslationLocale() throws ChameleonException;static 
   private native void CHMchameleonEncodingSetCurrentTranslationLocale(String NewLocale) throws ChameleonException;static 
   private native int CHMchameleonEncodingCountOfTranslationLocale() throws ChameleonException;static 
   private native String CHMchameleonEncodingTranslationLocale(int LocaleIndex) throws ChameleonException;
}