package com.interfaceware.chameleon;
/**
* This class handles Chameleon exceptions.  This is the
* standard exception object thrown by methods in the classes
* in this package.
**/
public class ChameleonException extends Exception
{
   private String m_Message;
   private long m_ErrorHandle;

 /**
 *  Construct an exception from a handle.
 *  This is used in the internal implementation of Chameleon and should not be called directly.
 **/
   public ChameleonException(long Handle) throws ChameleonException
   {
      super( "" );
      m_ErrorHandle = Handle;
      CHMchameleonExceptionAddRef(m_ErrorHandle);  
      m_Message = CHMchameleonExceptionGetMessage(m_ErrorHandle);      
   }

 /**
 *  Construct an exception from a String.
 *  This is used in the internal implementation of Chameleon and should not be called directly.
 **/
   public ChameleonException(String Message) throws ChameleonException
   {
      super( Message );
      m_Message = Message;
      m_ErrorHandle = CHMchameleonExceptionCreate(Message);
      CHMchameleonExceptionAddRef(m_ErrorHandle);
   }

   protected void finalize() throws ChameleonException
   {
      if (0 != m_ErrorHandle)
      {
         CHMchameleonExceptionRelease(m_ErrorHandle);
         m_ErrorHandle = 0;
      }
   }

 /**
 *  Return the exception message.
 **/
   public String getMessage()
   {
      return m_Message;
   }

 /**
 *  Get the value of an exception parameter.
 *  This returns an empty String if no such parameter is attached to the provided key. 
 *  In combination with {@link #countOfParameter() countOfParameter} and 
 *  {@link #getParameterKey(int) getParameterKey} this can be used to enumerate all 
 *  exception parameters.
 **/
   public String getParameter(String Key)
   {
      return CHMchameleonExceptionGetParameter(m_ErrorHandle,Key);
   }

 /**
  *  Get the number of parameters. In combination with 
  *  {@link #getParameterKey(int) getParameterKey} and {@link #getParameter(String) getParameter}
  *   this can be used to enumerate all exception parameters.
  **/
   public int countOfParameter()
   {
      return CHMchameleonExceptionGetCountOfParameter(m_ErrorHandle);
   }

 /**
  *  Get the parameter key by index. In combination with 
  *  {@link #countOfParameter() countOfParameter} and {@link #getParameter(String) getParameter}
  *   this can be used to enumerate all exception parameters.
  **/
   public String getParameterKey(int Index)
   {
      return CHMchameleonExceptionGetKey(m_ErrorHandle,Index);
   }

   static
   {
      System.loadLibrary("chm_java");
   }

   private native String CHMchameleonExceptionGetMessage(long Handle);
   private native long CHMchameleonExceptionCreate(String Message);
   private native void CHMchameleonExceptionAddRef(long Handle);
   private native void CHMchameleonExceptionRelease(long Handle);
   private native String CHMchameleonExceptionGetParameter(long Handle, String Key);
   private native int CHMchameleonExceptionGetCountOfParameter(long Handle);
   private native String CHMchameleonExceptionGetKey(long Handle,int Index);
}
