package com.interfaceware.chameleon;

//import java.util.Vector;
//import java.io.DataOutputStream;
//import java.io.DataInputStream;
//import java.io.IOException;
//import java.lang.Double;
//import java.lang.Integer;
/**
* This class allows access to programmatically change
* delimiter characters.  The Config object should not be created directly
* but instead should be obtained from the {@link Engine#config() Engine.config()} method.
* @see <a href="http://www.interfaceware.com/manual/delimiter_characters.html">http://www.interfaceware.com/manual/delimiter_characters.html</a>
**/
public class Config
{
/**
* Construct a Config object with the specified config handle.  This
* constructor is used in the internal implementation of Chameleon and should not be invoked directly.  
* To obtain a Config object use the {@link Engine#config() Engine.config()} method.
**/
   public Config(long Handle) throws ChameleonException
   {
      m_ConfigHandle = Handle;
      CHMconfigAddRef(m_ConfigHandle);
   }
   static
   {
      System.loadLibrary("chm_java");
   }
/**
* Free the Config object handle.
**/
   public void release() throws ChameleonException
   {
      CHMconfigRelease(m_ConfigHandle);
   }

/**
* Finalizer that invokes the release method.
**/
   protected void finalize() throws ChameleonException
   {
      release();
   }

/**
* Get the number of delimiter characters.
* @see <a href="http://www.interfaceware.com/manual/delimiter_characters.html">http://www.interfaceware.com/manual/delimiter_characters.html</a>
**/
   public int countOfDelimiter() throws ChameleonException
   {
      return CHMconfigCountOfDelimiter(m_ConfigHandle);
   }

/**
* Set the delimiter character at <code>CharIndex</code> to <code>NewValue</code>.
* @see <a href="http://www.interfaceware.com/manual/delimiter_characters.html">http://www.interfaceware.com/manual/delimiter_characters.html</a>
**/
   public void setDelimiterChar(int CharIndex, int NewValue) throws ChameleonException
   {
      CHMconfigSetDelimiterChar(m_ConfigHandle, CharIndex, NewValue);
   }
/**
* Get the delimiter character at <code>CharIndex</code>.
* @see <a href="http://www.interfaceware.com/manual/delimiter_characters.html">http://www.interfaceware.com/manual/delimiter_characters.html</a>
**/
   public int getDelimiterChar(int CharIndex) throws ChameleonException
   {
      return CHMconfigGetDelimiterChar(m_ConfigHandle, CharIndex);
   }

/**
* Set the escape character to <code>NewValue</code>.
* The default escape character is '\'.
* @see <a href="http://www.interfaceware.com/manual/delimiter_characters.html">http://www.interfaceware.com/manual/delimiter_characters.html</a>
**/
   public void setEscapeCharacter(int NewValue) throws ChameleonException
   {
      CHMconfigSetEscapeChar(m_ConfigHandle, NewValue);
   }

/**
* Get the escape character.
* The default escape character is '\'.
* @see <a href="http://www.interfaceware.com/manual/delimiter_characters.html">http://www.interfaceware.com/manual/delimiter_characters.html</a>
**/
   public int getEscapeCharacter() throws ChameleonException
   {
      return CHMconfigGetEscapeChar(m_ConfigHandle);
   }

/**
* Set the repeat character to <code>NewValue</code>.
* The default repeat character is '~'.
* @see <a href="http://www.interfaceware.com/manual/delimiter_characters.html">http://www.interfaceware.com/manual/delimiter_characters.html</a>
**/
   public void setRepeatCharacter(int NewValue) throws ChameleonException
   {
      CHMconfigSetRepeatChar(m_ConfigHandle, NewValue);
   }

/**
* Get the repeat character.
* The default repeat character is '~'.
* @see <a href="http://www.interfaceware.com/manual/delimiter_characters.html">http://www.interfaceware.com/manual/delimiter_characters.html</a>
**/
   public int getRepeatCharacter() throws ChameleonException
   {
      return CHMconfigGetRepeatChar(m_ConfigHandle);
   }

/**
* Get the Config object handle.  
* This is used in the internal implementation of Chameleon and should not be called directly.
**/
   public long handle()
   {
      return m_ConfigHandle;
   }

/**
* The Config object handle member variable.
**/
   private long m_ConfigHandle;

   private native void CHMconfigAddRef(long ConfigHandle) 
                           throws ChameleonException;

   private native void CHMconfigRelease(long ConfigHandle)
                           throws ChameleonException;

   private native int CHMconfigCountOfDelimiter
   (
      long ConfigHandle
   ) throws ChameleonException;

   private native void CHMconfigSetDelimiterChar
   (
      long ConfigHandle,
      int CharIndex,
      int NewValue
   ) throws ChameleonException;

   private native int CHMconfigGetDelimiterChar
   (
      long ConfigHandle,
      int CharIndex
   ) throws ChameleonException;

   private native void CHMconfigSetEscapeChar
   (
      long ConfigHandle,
      int NewValue
   ) throws ChameleonException;

   private native int CHMconfigGetEscapeChar
   (
      long ConfigHandle
   ) throws ChameleonException;

   private native void CHMconfigSetRepeatChar
   (
      long ConfigHandle,
      int NewValue
   ) throws ChameleonException;

   private native int CHMconfigGetRepeatChar
   (
      long ConfigHandle
   ) throws ChameleonException;
}
