package com.interfaceware.chameleon;
/**
* This class handles Message objects.
**/
public class Message extends Table
{
   private int m_MessageIndex;
   private Engine m_Engine;
   private boolean HasCalledGenerate;

/**
* Construct a Message object with the specified Engine and message index.
**/
   public Message(Engine AEngine, int MessageIndex) throws ChameleonException
   {
      // Call Table constructor
      super();
      HasCalledGenerate = false;
      m_Engine = AEngine;
      m_MessageIndex = MessageIndex;
      m_Engine.createEmptyMessageTable(this, MessageIndex);
      super.addRow();
   }
/**
* Construct a Message object with the specified Engine, message index, and table
* handle.  This is used in the internal implementation of Chameleon and should not
* be called directly.
**/
   public Message(Engine AEngine, int MessageIndex, long TableHandle) throws ChameleonException
   {
      super(TableHandle);        // copy table
      HasCalledGenerate = true;  // This is a parsed message
                                 // so the user should not be calling this
				 // method.
      m_MessageIndex = MessageIndex;
      m_Engine = AEngine;
   }

/**
* Free the Message object handle.
**/
   protected void finalize() throws ChameleonException
   {
      release();
   }

/**
* Get a Message object in text string format by calling the {@link Engine#generate(Table, int) Engine.generate}
* method.  This method will free the underlying table.
**/
   public String generateMessage() throws ChameleonException
   {
      if (HasCalledGenerate)
      {
         throw new ChameleonException("Illegal call to generateMessage.  Either this Message object was produced by parsing an object at which point it makes no sense to be invoking it or this is the second time you have invoked this function.");
      }
      String FlatWire = m_Engine.generate(this, m_MessageIndex);
      release();
      HasCalledGenerate = true;
      return FlatWire;
   }

/**
* Prevents adding a row to the Message object.
* Throws an exception when trying to add a row to the Message object because rows are not
* supposed to be added to the Message.
**/
   public void addRow() throws ChameleonException
   {
      throw new ChameleonException("addRow() method not used for message object.");
   }

}
