package com.interfaceware.chameleon;

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.*;

/**
 * This class is deprecated and should not be used for new applications.
 * This class implements a TCP/IP client for HL7 using the LLP protocol.  You
 * can use ClientSocketListener to track events on this object.
 * @see ClientSocketListener
 **/

public class TransportTcpIpClient extends Hl7ClientSocket implements Transport
{
   /** 
    * Default port
    **/
   public static final int DEFAULT_PORT = 8080;

   protected boolean m_IsConnected;

   private String   m_Host;
   protected int    m_Port;

   protected Log                   m_Log = null;
   protected ClientSocketListener  m_Listener = null;

   private Wrapper m_Wrapper = null;

   static
   {
      System.loadLibrary("chm_java");
   }
   
   /** 
    * Constructor
    * @see Wrapper
    **/
   public TransportTcpIpClient(int Port, String Host, Wrapper AWrapper)
   {
      m_Port = Port;
      m_Host = Host;
      m_Wrapper = AWrapper;
      if (0 == m_Port)
      {
         m_Port = DEFAULT_PORT;
      }
      try
      {
         connect( m_Host, m_Port );
      }
      catch (ChameleonException Error)
      {
         errorEvent( Error,"Error creating socket");
      }
   }

   /** 
    * Send an HL7 message to the server - the message needs to
    * presented as a string.
    **/
   public void sendMessage(String MessageData)
   {
      try
      {
         super.sendMessage( MessageData );
      }
      catch( ChameleonException Error )
      {
         errorEvent(Error,"write error");
      }
   }


   /** 
    * Closes the current connection
    **/
   public void close()
   {
      try
      {
         super.closeConnection();
      }
      catch( ChameleonException Error )
      {
         errorEvent(Error,"close error");
      }
   }

   /**
   * called when a message arrives
   **/
   public void onMessage( long TransportId, String Message )
   {
     StringBuffer MessageAsBuffer = new StringBuffer( Message );
     m_Wrapper.checkForMessage( MessageAsBuffer, this );
   }

   /** 
    * Print a line to the log.
    **/
   private void println(String Description)
   {
      if (m_Log != null)
      {
         m_Log.println(Description);
      }
   }

   /**
   * called when a connection has been established
   **/
   public void onConnect( long TransportId )
   {
      if (m_Listener != null)
      {
         m_Listener.onConnect( this );            
      }   
   }

   /**
   * called when an error occurs
   **/
   public void onTransportError( long TransportId, int ErrorId, String ErrorMessage )
   {
      if (m_Log != null)
      {
         m_Log.println(ErrorMessage);        
      }
      else
      {
         System.out.println(ErrorMessage);
      }
   }

   /** Invoke onError event **/
   private void errorEvent(Exception Error, String Description)
   {
      Error.printStackTrace();
      if (m_Log != null)
      {
         m_Log.println(Error.toString() + " : " + Description);        
      }
      else
      {
         System.out.println(Error.toString() + " : " + Description);
      }
   }

   /**
   * called when a connection closes
   **/
   public void onClose( long TransportId )
   {
      if (m_Listener != null)
      {
         m_Listener.onCloseClient(this);            
      }
   }

   /** 
    * Use this method to add a ClientSocketListener to
    * keep track on events happening on this socket.
    * @see ClientSocketListener
    **/
   public void addClientSocketListener(ClientSocketListener AListener)
   {
      m_Listener = AListener;
   }

   /** 
    * Use this method to assign the Log object for this 
    * socket to log errors to.
    * @see Log
    **/
   public void addLog(Log ALog)
   {
      m_Log = ALog;
   }
}

